#ifndef __F_ASCROLLVIEW_H__
#define __F_ASCROLLVIEW_H__

#include <qframe.h>
#include <qfontmetrics.h>

class QScrollBar : public QWidget
{
	Q_OBJECT
public:
	QScrollBar( Qt::Orientation eOrient, QWidget* pcParent, const char* pzName = NULL );
	~QScrollBar();
	void setValue(int val);
	int value();
	void setSteps(int small, int big);
	void setRange(int min, int max);
	virtual QSize sizeHint() const;
	os::ScrollBar* GetScrollBar();
	bool isShown() const;
private:
	class Private;
	Private* d;
};

class QScrollViewPrivate;

class QScrollView : public QFrame
{
    Q_OBJECT
public:
    
    QScrollView( QWidget* /*parent*/, const char* /*name*/, int /*flags*/ );
    ~QScrollView();

    virtual os::Point GetPreferredSize( bool bLargest ) const;
    virtual void HandleMessage( os::Message* pcMessage );
    virtual void WheelMoved( const os::Point& cDelta );
    virtual void MouseMove( const os::Point& cNewPos, int nCode, uint32 nButtons, os::Message* pcData );
    virtual void MouseDown( const os::Point& cPosition, uint32 nButtons );
    virtual void MouseUp( const os::Point& cPosition, uint32 nButtons, os::Message* pcData );
	virtual void KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
    virtual void KeyUp( const char* pzString, const char* pzRawString, uint32 nQualifiers );
    virtual void Paint( const os::Rect& cUpdateRect );
    virtual void TimerTick( int nID );
    
    QWidget* viewport();
    QWidget* clipper();


    void contentsToViewport(int x, int y, int& vx, int& vy);
    void viewportToContents(int vx, int vy, int& x, int& y);
    
    int	 contentsX();
    int	 contentsY();
    int  contentsWidth() const;
    int	 contentsHeight() const;
    
    QSize viewportSize( int x, int y );
    
    int	visibleWidth();
    int	visibleHeight();
    
    QScrollBar* horizontalScrollBar();
    QScrollBar* verticalScrollBar();
    virtual void setVScrollBarMode( ScrollBarMode );
    virtual void setHScrollBarMode( ScrollBarMode );

    void setStaticBackground( bool );

    bool focusNextPrevChild( bool /*next*/ );

	int childX( QWidget* widget );
	int childY( QWidget* widget );
    virtual void addChild( QWidget* /*child*/, int /*x*/=0, int /*y*/=0 );

    void enableClipper( bool );
    
    virtual void drawContents(QPainter*, int cx, int cy, int cw, int ch) = 0;
    void updateContents( QRect );
    void updateContents( int, int, int, int );
    void repaintContents( int x, int y, int w, int h, bool erase=true );
    void repaintContents( QRect );

public:
	virtual void resizeEvent ( QResizeEvent * event );
	virtual void viewportMousePressEvent( QMouseEvent * );
    virtual void viewportMouseMoveEvent(QMouseEvent *);
    virtual void viewportMouseReleaseEvent(QMouseEvent *);
    virtual void viewportResizeEvent(QResizeEvent*);
#ifndef QT_NO_WHEELEVENT
    virtual void viewportWheelEvent(QWheelEvent*);
#endif

    virtual void keyPressEvent( QKeyEvent *_ke );
    virtual void keyReleaseEvent ( QKeyEvent *_ke );
	
signals:
    void contentsMoving( int nX, int nY );    

public slots:
    virtual void resizeContents( int w, int h );
    void scrollBy( int, int );
    virtual void setContentsPos( int x, int y );
    void ensureVisible( int x, int y );
    void ensureVisible(int x, int y, int xmargin, int ymargin);
    
    
private:
    void UpdateScrollBars();
    
    QScrollViewPrivate* d;
};

#endif // __F_ASCROLLVIEW_H__











