#ifndef __F_AWIDGET_H__
#define __F_AWIDGET_H__

#include <gui/view.h>
#include <qrect.h>
#include <qobject.h>
#include <qpoint.h>
#include <qsize.h>
#include <qpalette.h>
#include <qpaintdevice.h>
#include <qnamespace.h>
#include <qtl.h>

class QWidgetPrivate;

class QPaintEvent;
class QFocusEvent;
class QFontMetrics;
class QStyle;
class QScrollBar;

class QWidget : public QObject, public os::View, public QPaintDevice
{
    Q_OBJECT
public:
    enum BackgroundMode { NoBackground };

    enum FocusPolicy {
	NoFocus = 0,
	TabFocus = 0x1,
	ClickFocus = 0x2,
	StrongFocus = 0x3,
	WheelFocus = 0x7
    };
    
    enum ScrollBarMode { Auto, AlwaysOff, AlwaysOn };

    
    QWidget( QWidget* pcParent = NULL, const char* pzName = NULL );
    virtual ~QWidget();

	WId winId() { return( (WId)this ); }
	QWidget* parentWidget();
	
    virtual void focusInEvent( QFocusEvent * );
    virtual void focusOutEvent( QFocusEvent * );
   	virtual void resizeEvent ( QResizeEvent * event );
    virtual void showEvent ( QShowEvent * );
    virtual void hideEvent ( QHideEvent *);
    virtual void closeEvent ( QCloseEvent * );
    
    virtual void setEnabled( bool bEnabled );
    virtual void setAutoMask(bool);
    
    void show();
    void hide();
    void raise();
    bool isVisible() const;
    
    int x();
    int y();
    QPoint	 pos()		const;
    int width();
    int height();
    QSize	 size()		const;
    const QRect &geometry()	const;
    const QRect &frameGeometry() { return( geometry() ); }
    int frameWidth();
    int frameHeight();
    virtual QSize sizeHint() const;    

    void		move( const QPoint & );
    void		move( int x, int y ) { move( QPoint( x, y ) ); }

    virtual void resize( int /*w*/, int /*h*/ );
    void		resize( const QSize & );
    
    QPoint mapToGlobal( const QPoint & ) const;
    QPoint mapFromGlobal( const QPoint& );

    int focusPolicy();
    virtual void setFocusPolicy( FocusPolicy );
    virtual void setFocusProxy( QWidget * );
    virtual bool focusNextPrevChild( bool next );
    QWidget* focusWidget() const;
    bool hasFocus();
    void setFocus();
    void clearFocus();
    
    QFontMetrics fontMetrics();
    virtual void setFont( const QFont & );
    QFont		font() const;

    virtual int	 metric( int id ) const;
      
    void		repaint();
   
    void 		constPolish() const;

    QStyle     &style() const;
    
    void setEraseColor( QColor color );
    
    void erase();
    const QPalette& palette() const;
    void setPalette( QPalette );
    void unsetPalette();
    
    QWidget* topLevelWidget();
    void setActiveWindow();
    
    virtual QScrollBar* horizontalScrollBar() { return( NULL ); }
    virtual QScrollBar* verticalScrollBar() { return( NULL ); }
    
    bool isTopLevel() const;
    bool isEnabled() const;
    bool isDefault() const;
    bool autoDefault() const;

    void setBackgroundMode( BackgroundMode );

    void setMouseTracking( bool );
    void setCursor( const QCursor& );
    
    virtual void setAcceptDrops( bool /*on*/ );

    virtual os::Point GetPreferredSize( bool bLargest ) const;
    virtual void Activated( bool bIsActive );
    virtual void FrameSized( const os::Point& cDelta );
    bool IsTopLevelWidget() const;
    void SetTopLevelWidget( bool );
private:
	void Lock();
	void Unlock();
    QWidgetPrivate* d;

};


#endif // __F_AWIDGET_H__








