#include <stdio.h>

#include <qcombobox.h>
#include <gui/dropdownmenu.h>
#include <util/message.h>

enum { ID_SELECTED };

class QComboBox::Private
{
public:
    os::DropdownMenu* m_pcMenu;
};


QComboBox::QComboBox( bool, QWidget* pcParent, const char *pzName ) : QWidget( pcParent, pzName )
{
    d = new Private;
    d->m_pcMenu = new os::DropdownMenu( GetBounds(), "combo_box", os::CF_FOLLOW_ALL );
    d->m_pcMenu->SetReadOnly( true );
    d->m_pcMenu->SetSelectionMessage( new os::Message( ID_SELECTED ) );
    AddChild( d->m_pcMenu );
    d->m_pcMenu->SetTarget( this );
}

QComboBox::~QComboBox()
{
	delete d;
}

int QComboBox::count() const
{
    return( d->m_pcMenu->GetItemCount() );
}


void QComboBox::popup()
{
   /* TODO: Change libsyllable to make this possible */
}

void QComboBox::clear()
{
    d->m_pcMenu->Clear();
}

void QComboBox::insertItem( const QString &text, int index )
{
    std::string cBuffer;

    if ( text.isEmpty() == false ) {
	cBuffer = text.ascii();
    }
    if ( index == -1 ) {
	d->m_pcMenu->AppendItem( text.ascii() );
    } else {
	d->m_pcMenu->InsertItem( index, cBuffer.c_str() );
    }
}

int	QComboBox::currentItem()
{
	return( d->m_pcMenu->GetSelection() );
}

void QComboBox::setCurrentItem( int index )
{
    d->m_pcMenu->SetSelection( index, false );
}

QSize QComboBox::sizeHint() const
{
    os::Point cSize = d->m_pcMenu->GetPreferredSize( false );
    return( QSize( (int)cSize.x, (int)cSize.y ) );
}


void QComboBox::HandleMessage( os::Message* pcMessage )
{
    switch( pcMessage->GetCode() )
    {
	case ID_SELECTED:
	{
	    bool bFinal = false;
	    pcMessage->FindBool( "final", &bFinal );
	    if ( bFinal ) {
		emit activated( d->m_pcMenu->GetSelection() );
	    }
	    break;
	}
	default:
	    QWidget::HandleMessage( pcMessage );
	    break;
    }
}

#include "qcombobox.moc"



