
#include <qlineedit.h>

#include <gui/textview.h>
#include <util/message.h>

enum { ID_SELECTED };

class QLineEdit::Private
{
public:
    os::TextView* m_pcTextView;
    bool m_bEdited;
};

QLineEdit::QLineEdit( QWidget* pcParent, const char* name ) : QWidget( pcParent, "text_view_cont" )
{
    d = new Private;
  	d->m_bEdited = false;
    d->m_pcTextView = new os::TextView( GetBounds(), "text_view", "", os::CF_FOLLOW_ALL );
    d->m_pcTextView->SetMessage( new os::Message( ID_SELECTED ) );
	if( name )
  		d->m_pcTextView->SetLabel( name );

    AddChild( d->m_pcTextView );
    d->m_pcTextView->SetTarget( this );
}


QLineEdit::~QLineEdit()
{
    delete d;
}

void QLineEdit::setSelection( int pos, int length )
{
	d->m_pcTextView->Select( os::IPoint( 0, pos ), os::IPoint( 0, pos + length ) );
}

bool QLineEdit::frame() const
{
    return( true );
}

void QLineEdit::selectAll()
{
    d->m_pcTextView->SelectAll();
}

void QLineEdit::insert( const QString& zString )
{
	d->m_pcTextView->Insert( zString.utf8().data() );
}

void QLineEdit::setDragEnabled( bool )
{
}

void QLineEdit::setAlignment( AlignmentFlags )
{
}

int QLineEdit::maxLength() const
{
    return( d->m_pcTextView->GetMaxLength() );
}

void QLineEdit::setMaxLength( int nLength )
{
    d->m_pcTextView->SetMaxLength( nLength );
}

void QLineEdit::setReadOnly( bool bFlag )
{
    d->m_pcTextView->SetReadOnly( bFlag );
}

QString QLineEdit::text() const
{
	return( QString::fromUtf8( d->m_pcTextView->GetBuffer()[0].c_str() ) );
}

void QLineEdit::setText( const QString& cText )
{
    d->m_pcTextView->Set( cText.utf8().data() );
}

bool QLineEdit::isReadOnly() const
{
    return( d->m_pcTextView->GetReadOnly() );
}

void QLineEdit::setEdited( bool bEdited )
{
	d->m_bEdited = bEdited;
}

bool QLineEdit::edited() const
{
	return( d->m_bEdited );
}

void QLineEdit::setCursorPosition( int nX )
{
	d->m_pcTextView->SetCursor( nX, 0 );
}

int QLineEdit::cursorPosition() const
{
	int nX, nY;
	d->m_pcTextView->GetCursor( &nX, &nY );
	return( nX );
}

void QLineEdit::setEchoMode( EchoMode eMode )
{
    d->m_pcTextView->SetPasswordMode( eMode != Normal );
}

QSize QLineEdit::sizeHint() const
{
    os::Point cSize = d->m_pcTextView->GetPreferredSize( false );
    return( QSize( (int)cSize.x + 1, (int)cSize.y + 1 ) );
}

void QLineEdit::SetMinPreferredSize( int nWidthChars )
{
    d->m_pcTextView->SetMinPreferredSize( nWidthChars, 1 );
    d->m_pcTextView->SetMaxPreferredSize( nWidthChars, 1 );
}

void QLineEdit::AllAttached()
{
    QWidget::AllAttached();
}

void QLineEdit::HandleMessage( os::Message* pcMessage )
{
    switch( pcMessage->GetCode() )
    {
	case ID_SELECTED:
	{
	    uint32 nEvents = 0;
	    
	    pcMessage->FindInt( "events", &nEvents );
	    if ( nEvents & os::TextView::EI_CONTENT_CHANGED ) {
		emit textChanged( QString::fromUtf8( d->m_pcTextView->GetBuffer()[0].c_str() ) );
	    }
	    if ( nEvents & os::TextView::EI_ENTER_PRESSED ) {
		emit returnPressed();
	    }
	    break;
	}
	default:
	    QWidget::HandleMessage( pcMessage );
	    break;
    }
}

#include "qlineedit.moc"


