
#include <stdio.h>

#include <qpalette.h>
#include <qcolor.h>
#include <gui/view.h>

class QColorGroupPrivate
{
public:
};

static QColor get_color( os::default_color_t eColor )
{
	os::Color32_s sColor = os::get_default_color( eColor );
	return( QColor( sColor.red, sColor.green, sColor.blue ) );
}

QColorGroup::QColorGroup()
{
	d = new QColorGroupPrivate;
	br = new QBrush[NColorRoles];
	br[Dark].setColor( QColor( 0, 0, 0 )  );
	br[Background].setColor( get_color( os::COL_NORMAL ) );
	br[Midlight].setColor( get_color( os::COL_NORMAL ) );
	br[Light].setColor( get_color( os::COL_SHINE ) );
	br[Base].setColor( QColor( 255, 255, 255 ) );
	br[Shadow].setColor( get_color( os::COL_SHADOW ) );
	br[Text].setColor( get_color( os::COL_MENU_TEXT ) );
	br[ButtonText].setColor( get_color( os::COL_MENU_TEXT ) );
	br[Button].setColor( get_color( os::COL_MENU_BACKGROUND ) );
	br[Highlight].setColor( QColor( 0, 0, 0 ) );
	br[HighlightedText].setColor( QColor( 255, 255, 255 ) );
	br[Link].setColor( QColor( 0, 0, 255 ) );
	br[LinkVisited].setColor( QColor( 255, 0, 0 ) );
}

QColorGroup::~QColorGroup()
{
	delete( d );
}

QColorGroup& QColorGroup::operator =(const QColorGroup&)
{
	return( *this );
}

void QColorGroup::setColor( ColorRole eRole, const QColor &cColor )
{
	br[eRole] = cColor;
}

const QColor& QColorGroup::color( ColorRole eRole ) const
{
	return( br[eRole].color() );
}


const QBrush& QColorGroup::brush( ColorRole eRole ) const
{
	return( br[eRole] );
}


QPalette::QPalette()
{
	data = new QPalData;
}

QPalette::QPalette( const QColor &/*button*/ )
{
	data = new QPalData;
}

QPalette::QPalette( const QColorGroup &active, const QColorGroup &disabled,
		    const QColorGroup &inactive )
{
	data = new QPalData;
	data->active = active;
	data->disabled = disabled;
	data->inactive = inactive;
}

QPalette::QPalette( const QPalette &p )
{
	
	data = new QPalData;
	data->active = p.data->active;
	data->disabled = p.data->disabled;
	data->inactive = p.data->inactive;
}

QPalette &QPalette::operator=( const QPalette &p )
{
	data->active = p.data->active;
	data->disabled = p.data->disabled;
	data->inactive = p.data->inactive;
    return *this;
}

QPalette::~QPalette()
{
	delete( data );
}

bool QPalette::operator==( const QPalette &/*p*/ ) const
{
	return( true );    
}

void QPalette::setColor( ColorGroup eGroup, QColorGroup::ColorRole eRole, const QColor &cColor )
{
	if( eGroup == Disabled )
		data->disabled.setColor( eRole, cColor );
	else if( eGroup == Active )
		data->active.setColor( eRole, cColor );
	else 
		data->inactive.setColor( eRole, cColor );
}

void QPalette::setColor( QColorGroup::ColorRole eRole, const QColor &cColor )
{
	data->disabled.setColor( eRole, cColor );
	data->active.setColor( eRole, cColor );
	data->inactive.setColor( eRole, cColor );
}

const QColor& QPalette::color( ColorGroup eGroup, QColorGroup::ColorRole eRole ) const
{
	if( eGroup == Disabled )
		return( data->disabled.color( eRole ) );
	else if( eGroup == Active )
		return( data->active.color( eRole ) );
	else 
		return( data->inactive.color( eRole ) );
}








