
#include <qpushbutton.h>

#include <gui/button.h>
#include <util/message.h>

enum { ID_SELECTED };

class QPushButtonWidget : public os::Button
{
public:
	QPushButtonWidget( os::Rect cFrame, os::String zName, os::String zLabel, os::Message* pcMsg, uint32 nResizeFlags )
		: os::Button( cFrame, zName, zLabel, pcMsg, nResizeFlags )
	{
	}
	virtual void  MouseMove( const os::Point& cNewPos, int nCode, uint32 nButtons, os::Message* pcData )
	{
		os::Button::MouseMove( cNewPos, nCode, nButtons, pcData );
		os::View::MouseMove( cNewPos, nCode, nButtons, pcData );
	}
    virtual void  MouseDown( const os::Point& cPosition, uint32 nButtons )
    {
    	os::Button::MouseDown( cPosition, nButtons );
    	os::View::MouseDown( cPosition, nButtons );    	
    }
    virtual void  MouseUp( const os::Point& cPosition, uint32 nButtons, os::Message* pcData )
    {
    	os::Button::MouseUp( cPosition, nButtons, pcData );
    	os::View::MouseUp( cPosition, nButtons, pcData );
    }
};

class QPushButton::Private
{
public:
    os::Button* m_pcButton;
};




QPushButton::QPushButton( QWidget* pcParent, const char* pzName ) : QWidget( pcParent, pzName )
{
    d = new Private;
    d->m_pcButton = new QPushButtonWidget( GetBounds(), "push_button", "Button", new os::Message( ID_SELECTED ), os::CF_FOLLOW_ALL );
    AddChild( d->m_pcButton );
    d->m_pcButton->SetTarget( this );
}

QPushButton::QPushButton( const QString &text, QWidget* pcParent, const char* /*name*/ ) : QWidget( pcParent, "push_button_cont" )
{
    d = new Private;
    d->m_pcButton = new os::Button( GetBounds(), "push_button", text.ascii(), NULL, os::CF_FOLLOW_ALL );
    AddChild( d->m_pcButton );
    d->m_pcButton->SetTarget( this );
}

QPushButton::~QPushButton()
{
    delete d;
}

void QPushButton::AllAttached()
{
    QWidget::AllAttached();
}

void QPushButton::HandleMessage( os::Message* pcMessage )
{
    switch( pcMessage->GetCode() )
    {
	case ID_SELECTED:
	    emit clicked();
	    break;
	default:
	    QWidget::HandleMessage( pcMessage );
	    break;
    }
}

void QPushButton::setEnabled( bool bEnabled )
{
	d->m_pcButton->SetEnable( bEnabled );
}

void QPushButton::setText( const QString& cString )
{
    d->m_pcButton->SetLabel( cString.ascii() );
}

QString QPushButton::text()
{
	return( d->m_pcButton->GetLabel().c_str() );
}

QSize QPushButton::sizeHint() const
{
    os::Point cSize = d->m_pcButton->GetPreferredSize( false );
    return( QSize( cSize.x + 1, cSize.y + 1 ) );
}

#include "qpushbutton.moc"




