
#include <stdio.h>
#include <gui/region.h>
#include "qregion.h"

class QRegion::Private
{
public:
	os::Region* m_pcReg;
};

QRegion::QRegion()
{
	d = new Private;
	d->m_pcReg = new os::Region();
}

QRegion::QRegion( int x, int y, int w, int h, RegionType eType )
{
	d = new Private;
	d->m_pcReg = new os::Region( os::IRect( x, y, x + w - 1, y + h - 1 ) );
	if( eType != QRegion::Rectangle )
	{
		printf( "Error: QRegion::QRegion(): Non-rectangular areas not supported\n" );
	}
}

QRegion::QRegion( const QPointArray &, bool winding )
{
	d = new Private;
	d->m_pcReg = new os::Region();
	printf( "Error: QRegion::QRegion(): Point arrays not supported\n" );
}

QRegion::QRegion( const QRegion &cRegion )
{
	d = new Private;
	d->m_pcReg = new os::Region();
	d->m_pcReg->Set( *cRegion.d->m_pcReg );
}

QRegion::~QRegion()
{
	delete( d->m_pcReg );
	delete( d );
}

QRegion& QRegion::operator=( const QRegion &cOther )
{
	d->m_pcReg->Set( *cOther.d->m_pcReg );
    return( *this );
}

QRegion QRegion::intersect( const QRegion &cOther ) const
{
	d->m_pcReg->Intersect( *cOther.d->m_pcReg );
    return( *this );
}

QRegion QRegion::unite( const QRect& rect ) const
{
	d->m_pcReg->Include( os::IRect( rect.left(), rect.top(), rect.right(), rect.bottom() ) );
    return( *this );
}

void QRegion::translate( int dx, int dy )
{
	os::ClipRect* pcRect;
	ENUMCLIPLIST( &d->m_pcReg->m_cRects, pcRect )
	{
		pcRect->m_cBounds += os::IPoint( dx, dy );
	}
}

QRect QRegion::boundingRect() const
{
	os::IRect cRect = d->m_pcReg->GetBounds();
	
	return( QRect( cRect.left, cRect.top, cRect.Width() + 1, cRect.Height() + 1 ) );
}

bool QRegion::isNull()
{
	return( d->m_pcReg->IsEmpty() );
}

bool QRegion::contains( const QPoint &p ) const
{
	os::IPoint cPoint( p.x(), p.y() );
	os::ClipRect* pcRect;
	ENUMCLIPLIST( &d->m_pcReg->m_cRects, pcRect )
	{
		if( pcRect->m_cBounds.DoIntersect( cPoint ) )
			return( true );
	}
    return( false );
}










