#include <stdio.h>

#include <qpoint.h>
#include "qscrollview.h"
#include "qpainter.h"

#include <gui/scrollbar.h>
#include <util/message.h>

enum { ID_VSCROLL, ID_HSCROLL };

class QScrollBar::Private
{
public:
	os::ScrollBar* m_pcScroll;
};

QScrollBar::QScrollBar( Qt::Orientation eOrient, QWidget* pcParent, const char* pzName ) : QWidget( pcParent, pzName )
{
	d = new Private;
	if( eOrient == Qt::Horizontal ) {
		SetResizeMask( os::CF_FOLLOW_BOTTOM | os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT );
		d->m_pcScroll = new os::ScrollBar( GetBounds(), "hscroll", new os::Message( ID_HSCROLL ), 0, 0, os::HORIZONTAL,
						   os::CF_FOLLOW_ALL );
	} else {
		SetResizeMask( os::CF_FOLLOW_BOTTOM | os::CF_FOLLOW_TOP | os::CF_FOLLOW_RIGHT );
		d->m_pcScroll = new os::ScrollBar( GetBounds(), "vscroll", new os::Message( ID_VSCROLL ), 0, 0, os::VERTICAL,
						   os::CF_FOLLOW_ALL );
	}
	AddChild( d->m_pcScroll ); 
}

QScrollBar::~QScrollBar()
{
	delete( d );
}

void QScrollBar::setValue( int val )
{
	d->m_pcScroll->SetValue( val );
}
int QScrollBar::value()
{
	return( d->m_pcScroll->GetValue().AsInt32() );
}
void QScrollBar::setSteps( int small, int big )
{
	d->m_pcScroll->SetSteps( small, big );
}

void QScrollBar::setRange( int min, int max )
{
	d->m_pcScroll->SetMinMax( min, max );
}
	
QSize QScrollBar::sizeHint() const
{
	if( IsVisible() )
	{
	    os::Point cSize = d->m_pcScroll->GetPreferredSize( false );
	    return( QSize( (int)cSize.x + 1, (int)cSize.y + 1 ) );
	} else
		return( QSize( 0, 0 ) );
}
	
os::ScrollBar* QScrollBar::GetScrollBar()
{
	return( d->m_pcScroll );
}

bool QScrollBar::isShown() const
{
	return( isVisible() );
}


class ViewPort : public QWidget
{
public:
    ViewPort( QScrollView* pcParent ) : QWidget( pcParent, "sv_viewport" ) { m_pcParent = pcParent; }

    virtual void Paint( const os::Rect& cUpdateRect )
    {
	QPainter cPainter( this );
	m_pcParent->drawContents( &cPainter, (int)cUpdateRect.left, (int)cUpdateRect.top, (int)cUpdateRect.Width() + 1, (int)cUpdateRect.Height() + 1 );
    }
private:
    QScrollView* m_pcParent;
    
};

class QScrollViewPrivate
{
public:
    ViewPort*		       m_pcViewPort;
    QScrollBar*		       m_pcHScrollBar;
    QScrollBar*		       m_pcVScrollBar;
    QScrollView::ScrollBarMode m_eVScrollBarMode;
    QScrollView::ScrollBarMode m_eHScrollBarMode;
    int						m_nContentWidth;
    int						m_nContentHeight;
    bool					m_bMouseDownSeen;
};



QScrollView::QScrollView( QWidget* pcParent, const char* pzName, int /*flags*/ ) : QFrame( pcParent, pzName )
{
	printf("QScrollView::QScrollView(%s)\n", GetName().c_str() );
    d = new QScrollViewPrivate;
    d->m_bMouseDownSeen = false;
    d->m_pcHScrollBar = new QScrollBar( Qt::Horizontal, this, "scrollbar" );
    d->m_pcVScrollBar = new QScrollBar( Qt::Vertical, this, "scrollbar" );
    d->m_pcViewPort = NULL;
    d->m_pcViewPort = new ViewPort( this );
    d->m_pcViewPort->SetResizeMask( os::CF_FOLLOW_ALL );
    d->m_pcViewPort->SetFrame( GetBounds() /*os::Rect( 0.0f, 0.0f, 100000.0f, 100000.0f )*/ );
    d->m_eHScrollBarMode = Auto;
    d->m_eVScrollBarMode = Auto;
    d->m_nContentWidth  = 0;
    d->m_nContentHeight = 0;
    
    
}

QScrollView::~QScrollView()
{
    delete d;
}

QWidget* QScrollView::viewport()
{
    return( d->m_pcViewPort );
}

QWidget* QScrollView::clipper()
{
	/* We do not have a clipper view -> return ourself */
	return( this );
}

void QScrollView::resizeEvent ( QResizeEvent * event )
{
	viewportResizeEvent( event );
}

void QScrollView::viewportMousePressEvent( QMouseEvent * )
{
}

void QScrollView::viewportMouseMoveEvent(QMouseEvent *)
{
}

void QScrollView::viewportMouseReleaseEvent(QMouseEvent *)
{
}

void QScrollView::viewportResizeEvent(QResizeEvent*)
{
}

#ifndef QT_NO_WHEELEVENT
void QScrollView::viewportWheelEvent(QWheelEvent* pcEvent)
{
	if( d->m_pcVScrollBar->isVisible()/* != NULL*/ )
	    /*d->m_pcVScrollBar->GetScrollBar()->*/scrollBy( 0, pcEvent->delta() );
}
#endif

void QScrollView::keyPressEvent( QKeyEvent *_ke )
{
}

void QScrollView::keyReleaseEvent ( QKeyEvent *_ke )
{
}

void QScrollView::setContentsPos( int x, int y )
{
    if ( x < 0 ) {
		x = 0;
    } else if ( x > contentsWidth() - visibleWidth() ) {
	    if( visibleWidth() > contentsWidth() )
	    	x = 0;
	    else
			x = contentsWidth() - visibleWidth();
    }
    if ( y < 0 ) {
		y = 0;
    } else if ( y > contentsHeight() - visibleHeight() ) {
    	if( visibleHeight() > contentsHeight() )
    		y = 0;
    	else
			y = contentsHeight() - visibleHeight();
    }
    d->m_pcViewPort->ScrollTo( -x, -y );
   // Flush();
    UpdateScrollBars();
}

void QScrollView::resizeContents( int w, int h )
{
//    d->m_pcViewPort->ResizeTo( w - 1, h - 1 );
    d->m_nContentWidth  = w;
    d->m_nContentHeight = h;
    
    os::Point cOffset = -d->m_pcViewPort->GetScrollOffset();

    if ( w < visibleWidth() ) {
	cOffset.x = 0.0f;
    } else if ( cOffset.x > w - visibleWidth() ) {
	cOffset.x = w - visibleWidth();
    }
    
    if ( h < visibleHeight() ) {
	cOffset.y = 0.0f;
    } else if ( cOffset.y > h - visibleHeight() ) {
	cOffset.y = h - visibleHeight();
	
    }
    d->m_pcViewPort->ScrollTo( -cOffset );
    //Flush();
    UpdateScrollBars();
}

void QScrollView::scrollBy( int x, int y )
{
	int nNewX = contentsX() + x;
	int nNewY = contentsY() + y;
	if( nNewX < 0 ) nNewX = 0;
	if( nNewY < 0 ) nNewY = 0;
    setContentsPos( nNewX, nNewY );
}

void QScrollView::contentsToViewport(int x, int y, int& vx, int& vy)
{
    vx = x - contentsX();
    vy = y - contentsY();
}

void QScrollView::viewportToContents(int vx, int vy, int& x, int& y)
{
    x = vx + contentsX();
    y = vy + contentsY();
}

int QScrollView::contentsX()
{
    return( -int(d->m_pcViewPort->GetScrollOffset().x) );
}

int QScrollView::contentsY()
{
    return( -int(d->m_pcViewPort->GetScrollOffset().y) );
}

int QScrollView::contentsWidth() const
{
    return( d->m_nContentWidth );
//    return( int(d->m_pcViewPort->GetFrame().Width()) + 1 );
}

int QScrollView::contentsHeight() const
{
    return( d->m_nContentHeight );
//    return( int(d->m_pcViewPort->GetFrame().Height()) + 1 );
}

void QScrollView::ensureVisible( int x, int y )
{
    ensureVisible( x, y, 0, 0 );
}

void QScrollView::ensureVisible(int x, int y, int xmargin, int ymargin)
{
    int nOldX = contentsX();
    int nOldY = contentsY();
    
    if ( x < nOldX + xmargin ) {
	nOldX = x - xmargin;
    }
    if ( x > nOldX + visibleWidth() - xmargin ) {
	nOldX = x - visibleWidth() + xmargin;
    }
    if ( y < nOldY + ymargin ) {
	nOldY = y - ymargin;
    }
    if ( y > nOldY + visibleHeight() - ymargin ) {
	nOldY = y - visibleHeight() + ymargin;
    }
    setContentsPos( nOldX, nOldY );
}

    
int QScrollView::visibleWidth()
{
	int nWidth = (int)d->m_pcViewPort->GetFrame().Width() + 1;
    return( nWidth );
//    return( int(d->m_pcViewPort->GetFrame().Width()) + 1 );
//    return( int(GetFrame().Width()) + 1 );
}

int QScrollView::visibleHeight()
{
	int nHeight = (int)d->m_pcViewPort->GetFrame().Height() + 1;
    return( nHeight );
//    return( int(GetFrame().Height()) + 1 );
}

QSize QScrollView::viewportSize( int x, int y )
{
	bool bShowH = false;
	bool bShowV = false;
	bool bNeedH = width() < x;
	bool bNeedV = height() < y;
	
	if( d->m_eHScrollBarMode == AlwaysOn )
		bShowH = true;
	else if( d->m_eHScrollBarMode == AlwaysOff )
		bShowH = false;
	else
		bShowH = bNeedH;
	
	if( d->m_eVScrollBarMode == AlwaysOn )
		bShowV = true;
	else if( d->m_eVScrollBarMode == AlwaysOff )
		bShowV = false;
	else
		bShowV = bNeedV;
		
	
	return( QSize( width() - ( bShowV ? (int)d->m_pcVScrollBar->GetScrollBar()->GetPreferredSize(false).x : 0 ), 
		height() - ( bShowH ? (int)d->m_pcHScrollBar->GetScrollBar()->GetPreferredSize(false).y : 0 ) ) );

}

int childX( QWidget* widget );
int childY( QWidget* widget );

QScrollBar* QScrollView::horizontalScrollBar()
{
	return( d->m_pcHScrollBar );
}

QScrollBar* QScrollView::verticalScrollBar()
{
	return( d->m_pcVScrollBar );
}

void QScrollView::setVScrollBarMode( ScrollBarMode eMode )
{
    d->m_eVScrollBarMode = eMode;
    UpdateScrollBars();
}

void QScrollView::setHScrollBarMode( ScrollBarMode eMode )
{
    d->m_eHScrollBarMode = eMode;
    UpdateScrollBars();
}

void QScrollView::setStaticBackground( bool )
{
    //printf( "Warning: QScrollView::%s() not implemented\n", __FUNCTION__ );
}


bool QScrollView::focusNextPrevChild( bool /*next*/ )
{
    printf( "Warning: QScrollView::%s() not implemented\n", __FUNCTION__ );
    return( false );
}


int QScrollView::childX( QWidget* widget )
{
	return( (int)widget->GetFrame().left + contentsX() );
}



int QScrollView::childY( QWidget* widget )
{
	return( (int)widget->GetFrame().top + contentsY() );
}
void QScrollView::addChild( QWidget* child, int x, int y )
{
    if ( child->GetWindow() != GetWindow() ) {
	d->m_pcViewPort->AddChild( child );
    }
    child->MoveTo( x, y );
}

void QScrollView::enableClipper( bool )
{
//    printf( "Warning: QScrollView::%s() not implemented\n", __FUNCTION__ );
}


void QScrollView::updateContents( QRect rect )
{
    d->m_pcViewPort->Invalidate( os::Rect( rect.left(), rect.top(), rect.right(), rect.bottom() ) );
    d->m_pcViewPort->Flush();
}

void QScrollView::updateContents( int x, int y, int w, int h )
{
    d->m_pcViewPort->Invalidate( os::Rect( x, y, x+w-1, y+h-1 ) );
    d->m_pcViewPort->Flush();
}

void QScrollView::repaintContents( int x, int y, int w, int h, bool /*erase*/ )
{
//    printf( "Warning: QScrollView::%s() not implemented\n", __FUNCTION__ );
    //QPainter cPainter( d->m_pcViewPort );

    //drawContents( &cPainter, x, y, w, h );
    d->m_pcViewPort->Invalidate( os::Rect( x, y, x+w-1, y+h-1 ) );
    d->m_pcViewPort->Flush();
}

void QScrollView::repaintContents( QRect rect )
{
	//QPainter cPainter( d->m_pcViewPort );
//	drawContents( &cPainter, rect.left(), rect.top(), rect.right(), rect.bottom() );
	d->m_pcViewPort->Invalidate( os::Rect( rect.left(), rect.top(), rect.right(), rect.bottom() ) );
    d->m_pcViewPort->Flush();
}

void QScrollView::Paint( const os::Rect& cUpdateRect )
{
    FillRect( cUpdateRect, os::Color32_s( 255, 255, 255 ) );
    QPainter cPainter( d->m_pcViewPort );
    drawContents( &cPainter, (int)cUpdateRect.left, (int)cUpdateRect.top, (int)cUpdateRect.Width() + 1, (int)cUpdateRect.Height() + 1 );

}

void QScrollView::UpdateScrollBars()
{
    if ( !d->m_pcVScrollBar->isVisible() ) {
	if ( (d->m_eVScrollBarMode == Auto && visibleHeight() < contentsHeight()) || d->m_eVScrollBarMode == AlwaysOn ) {
//	    d->m_pcVScrollBar = new os::ScrollBar( os::Rect(0,0,0,0), "vscroll", new os::Message( ID_VSCROLL ), 0, 0, os::VERTICAL,
//						   os::CF_FOLLOW_RIGHT | os::CF_FOLLOW_TOP | os::CF_FOLLOW_BOTTOM );

	    os::Rect cFrame = GetBounds();
	    cFrame.left = cFrame.right - d->m_pcVScrollBar->GetScrollBar()->GetPreferredSize(false).x;
	    if ( d->m_pcHScrollBar->isVisible()/* != NULL*/ ) {
		cFrame.bottom -= d->m_pcHScrollBar->GetBounds().Height() + 1;
	    }
	    d->m_pcVScrollBar->SetFrame( cFrame );
	    d->m_pcVScrollBar->show();
//	    AddChild( d->m_pcVScrollBar );
	    d->m_pcVScrollBar->GetScrollBar()->SetTarget( this );
	    if ( d->m_pcHScrollBar->isVisible()/* != NULL*/ ) {
		d->m_pcHScrollBar->ResizeBy( -(cFrame.Width() + 1), 0 );
	    }
	    
	    d->m_pcViewPort->ResizeBy( -(cFrame.Width() + 1), 0 );
	}
    } else {
	if ( (d->m_eVScrollBarMode == Auto && visibleHeight() >= contentsHeight()) || d->m_eVScrollBarMode == AlwaysOff ) {
	    if ( d->m_pcHScrollBar->isVisible()/* != NULL*/ ) {
		d->m_pcHScrollBar->ResizeBy( d->m_pcVScrollBar->GetBounds().Width() + 1, 0 );
	    }
	    d->m_pcViewPort->ResizeBy( d->m_pcVScrollBar->GetBounds().Width() + 1, 0 );
	    //RemoveChild( d->m_pcVScrollBar );
	    //delete d->m_pcVScrollBar;
	    //d->m_pcVScrollBar = NULL;
	    d->m_pcVScrollBar->hide();
	}
    }
    if ( !d->m_pcHScrollBar->isVisible()/* == NULL*/ ) {
	if ( (d->m_eHScrollBarMode == Auto && visibleWidth() < contentsWidth()) || d->m_eHScrollBarMode == AlwaysOn ) {
//	    d->m_pcHScrollBar = new os::ScrollBar( os::Rect(0,0,0,0), "hscroll", new os::Message( ID_HSCROLL ), 0, 0, os::HORIZONTAL,
//						   os::CF_FOLLOW_BOTTOM | os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT );
	    os::Rect cFrame = GetBounds();
	    cFrame.top = cFrame.bottom - d->m_pcHScrollBar->GetScrollBar()->GetPreferredSize(false).y;
	    if ( d->m_pcVScrollBar->isVisible()/* != NULL*/ ) {
		cFrame.right -= d->m_pcVScrollBar->GetBounds().Width() + 1;
	    }
	    d->m_pcHScrollBar->SetFrame( cFrame );
	    d->m_pcHScrollBar->show();
//	    AddChild( d->m_pcHScrollBar );
	    d->m_pcHScrollBar->GetScrollBar()->SetTarget( this );

	    if ( d->m_pcVScrollBar->isVisible()/* != NULL*/ ) {
		d->m_pcVScrollBar->ResizeBy( 0, -(cFrame.Height() + 1) );
	    }
	    d->m_pcViewPort->ResizeBy( 0, -(cFrame.Height() + 1) );
	}
    } else {
	if ( (d->m_eHScrollBarMode == Auto && visibleWidth() >= contentsWidth()) || d->m_eHScrollBarMode == AlwaysOff ) {
	    if ( d->m_pcVScrollBar->isVisible()/* != NULL*/ ) {
	    	d->m_pcVScrollBar->ResizeBy( 0, d->m_pcHScrollBar->GetBounds().Height() + 1 );
	    }
	    d->m_pcViewPort->ResizeBy( 0, d->m_pcHScrollBar->GetBounds().Height() + 1 );
//	    RemoveChild( d->m_pcHScrollBar );
//	    delete d->m_pcHScrollBar;
//	    d->m_pcHScrollBar = NULL;
		d->m_pcHScrollBar->hide();
	}
    }

    if ( d->m_pcHScrollBar->isVisible()/* != NULL*/ ) {
	d->m_pcHScrollBar->GetScrollBar()->SetMinMax( 0, contentsWidth() - visibleWidth() );
	d->m_pcHScrollBar->GetScrollBar()->SetValue( contentsX(), false );
	d->m_pcHScrollBar->GetScrollBar()->SetProportion( float(visibleWidth()) / float(contentsWidth()) );
	d->m_pcHScrollBar->GetScrollBar()->SetSteps( 10.0f, float(visibleWidth()) * 0.8f );
    }
    if ( d->m_pcVScrollBar->isVisible() /*!= NULL*/ ) {
	d->m_pcVScrollBar->GetScrollBar()->SetMinMax( 0, contentsHeight() - visibleHeight() );
	d->m_pcVScrollBar->GetScrollBar()->SetValue( contentsY(), false );
	d->m_pcVScrollBar->GetScrollBar()->SetProportion( float(visibleHeight()) / float(contentsHeight()) );
	d->m_pcVScrollBar->GetScrollBar()->SetSteps( 10.0f, float(visibleHeight()) * 0.8f );
    }
}



void QScrollView::HandleMessage( os::Message* pcMessage )
{
    switch( pcMessage->GetCode() )
    {
	case ID_HSCROLL:
	    if ( d->m_pcHScrollBar->isVisible()/* != NULL*/ ) {
		setContentsPos( d->m_pcHScrollBar->GetScrollBar()->GetValue(), contentsY() );
		emit contentsMoving( contentsX(), contentsY() );
	    }
	    break;
	case ID_VSCROLL:
	    if ( d->m_pcVScrollBar->isVisible()/* != NULL*/ ) {
		setContentsPos( contentsX(), d->m_pcVScrollBar->GetScrollBar()->GetValue() );
		emit contentsMoving( contentsX(), contentsY() );
	    }
	    break;
    }
}

static int convert_button_state( uint32 nButtons, uint32 nQualifiers )
{
    int nButtonState = 0;
    
    if ( nButtons & 0x01 ) {
	nButtonState |= Qt::LeftButton;
    }
    else if ( nButtons & 0x02 ) {
	nButtonState |= Qt::RightButton;
    }
    else if ( nButtons & 0x04 ) {
	nButtonState |= Qt::MidButton;
    }
    
    if ( nQualifiers & os::QUAL_SHIFT ) {
	nButtonState |= Qt::ShiftButton;
    }
    if ( nQualifiers & os::QUAL_CTRL ) {
	nButtonState |= Qt::ControlButton;
    }
    if ( nQualifiers & os::QUAL_ALT ) {
	nButtonState |= Qt::AltButton;
    }
    return( nButtonState );
}

static int convert_key_code( char cChar )
{
	switch( cChar )
    {
	    case ' ':
			return( Qt::Key_Space );
		break;
	    case os::VK_DOWN_ARROW:
			return( Qt::Key_Down );
		break;
	    case os::VK_UP_ARROW:
			return( Qt::Key_Up );
		break;
	    case os::VK_RIGHT_ARROW:
			return( Qt::Key_Right );
		break;
	    case os::VK_LEFT_ARROW:
			return( Qt::Key_Left );
		break;
	    case os::VK_PAGE_DOWN:
			return( Qt::Key_PageDown );
		break;
	    case os::VK_PAGE_UP:
			return( Qt::Key_PageUp );
		break;
	    case os::VK_ENTER:
			return( Qt::Key_Enter );
		break;
	    case os::VK_HOME:
			return( Qt::Key_Home );
		break;
	    case os::VK_END:
			return( Qt::Key_End );
		break;
    }
	return( 0 );
}

void QScrollView::MouseMove( const os::Point& cNewPos, int nCode, uint32 nButtons, os::Message* pcData )
{
    int nButtonState = convert_button_state( nButtons, GetQualifiers() );
    
    os::Point cGlobalPos = ConvertToScreen( cNewPos );
    QMouseEvent cEvent( QEvent::MouseMove, QPoint( (int)cNewPos.x, (int)cNewPos.y ), QPoint( (int)cGlobalPos.x, (int)cGlobalPos.y ), 0, nButtonState );
    event( &cEvent );
    viewportMouseMoveEvent( &cEvent );
    
    QFrame::MouseMove( cNewPos, nCode, nButtons, pcData );
}

void QScrollView::MouseDown( const os::Point& cPosition, uint32 nButtonOrig )
{
    d->m_bMouseDownSeen = true;
    MakeFocus( true );
    uint32 nButtons;
    
    GetMouse( NULL, &nButtons );
	
    int nButtonState = convert_button_state( nButtons, GetQualifiers() );
	int nButton = convert_button_state( 1 << (nButtonOrig-1), 0 );

    os::Point cGlobalPos = ConvertToScreen( cPosition );
    QMouseEvent cEvent( QEvent::MouseButtonPress, QPoint( (int)cPosition.x, (int)cPosition.y ), QPoint( (int)cGlobalPos.x, (int)cGlobalPos.y ), nButton, nButtonState );
    event( &cEvent );
    viewportMousePressEvent( &cEvent );
    
    QFrame::MouseDown( cPosition, nButtonOrig );
}

void QScrollView::MouseUp( const os::Point& cPosition, uint32 nButtonOrig, os::Message* pcData )
{
    uint32 nButtons;

    if ( d->m_bMouseDownSeen == false ) {
	return;
    }
    d->m_bMouseDownSeen = false;
    GetMouse( NULL, &nButtons );
	
    int nButtonState = convert_button_state( nButtons, GetQualifiers() );
    int nButton = convert_button_state( 1 << (nButtonOrig-1), 0 );
    
    os::Point cGlobalPos = ConvertToScreen( cPosition );
    QMouseEvent cEvent( QEvent::MouseButtonRelease, QPoint( (int)cPosition.x, (int)cPosition.y ), QPoint( (int)cGlobalPos.x, (int)cGlobalPos.y ), nButton, nButtonState );
	event( &cEvent );
    viewportMouseReleaseEvent( &cEvent );
    
    QFrame::MouseUp( cPosition, nButton, pcData );
}

void QScrollView::KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if( !pzRawString )
		return( QFrame::KeyDown( pzString, pzRawString, nQualifiers ) );
	int nCode = convert_key_code( pzRawString[0] );
	if( nCode != 0 )
	{
		QKeyEvent cEvent( QEvent::KeyPress, nCode, 0, convert_button_state( 0, nQualifiers ) );
		event( &cEvent );
		keyPressEvent( &cEvent );
	}
	QFrame::KeyDown( pzString, pzRawString, nQualifiers );
}

void QScrollView::KeyUp( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if( !pzRawString )
		return( QFrame::KeyUp( pzString, pzRawString, nQualifiers ) );
	int nCode = convert_key_code( pzRawString[0] );
	if( nCode != 0 )
	{
		QKeyEvent cEvent( QEvent::KeyPress, nCode, 0, convert_button_state( 0, nQualifiers ) );
		event( &cEvent );
		keyReleaseEvent( &cEvent );
	}
	QFrame::KeyUp( pzString, pzRawString, nQualifiers );

}

void QScrollView::WheelMoved( const os::Point& cDelta )
{
#ifndef QT_NO_WHEELEVENT
	uint32 nQual = GetQualifiers();
	QWheelEvent cEvent( QPoint( 0, 0 ), (int)( cDelta.y * 30 ), convert_button_state( 0, nQual ) );

	event( &cEvent );
	viewportWheelEvent( &cEvent );
#endif
	QFrame::WheelMoved( cDelta );
}


void QScrollView::TimerTick( int nID )
{
	QTimerEvent cEvent( nID );
	timerEvent( &cEvent );
}

os::Point QScrollView::GetPreferredSize( bool bLargest ) const
{
    return( (bLargest) ? os::Point( 1000000.0f, 1000000.0f ) : os::Point( 0.0f, 0.0f ) );
}

#include "qscrollview.moc"

























