
#include <qtimer.h>
#include "qmutex.h"
#include <util/looper.h>
#include <util/application.h>
#include <cassert>

#define g_pcTimerLooper (os::Application::GetInstance())

enum { ID_TIMER };
#define TIMER_COUNT 50
static QTimer* g_pcTimerList[TIMER_COUNT];


QTimer::QTimer( QObject *parent, const char *name ) : QObject( parent, name ), Handler( "timer" )
{
    m_bIsActive = false;
    m_bDeleteWhenTrigged = false;
    m_bSingleShot        = false;
    m_bIsObjectTimer	 = false;
    m_pcObject			 = NULL;
    m_nObjectTimerID	 = 0;
    g_pcTimerLooper->AddHandler( this );
}
    
QTimer::~QTimer()
{
    if ( m_bIsActive ) {
    	if( m_bIsObjectTimer )
    	{
	    	assert( g_pcTimerLooper->RemoveTimer( this, m_nObjectTimerID ) == true );
	    }
    	else {
	   		assert( g_pcTimerLooper->RemoveTimer( this, ID_TIMER ) == true );
	   	}
    }
    g_pcTimerLooper->RemoveHandler( this );
}

void QTimer::TimerTick( int nID )
{
	if( m_bIsObjectTimer )
	{
		assert( m_nObjectTimerID == nID );
		QTimerEvent cEvent( nID );
		m_pcObject->event( &cEvent );
		/* Return as fast as possible -> the timer event might have deleted us */
		return;
	} else {
	    emit timeout();
	}
    if ( m_bSingleShot ) {
	m_bIsActive = false;
    }
    if ( m_bDeleteWhenTrigged ) {
	delete this;
    }
}

bool QTimer::isActive() const
{
    return( m_bIsActive );
}

int QTimer::start( int msec, bool sshot )
{
    m_bIsActive = true;
    m_bSingleShot = sshot;
    g_pcTimerLooper->Lock();
    g_pcTimerLooper->AddTimer( this, ID_TIMER, bigtime_t(msec) * 1000LL, sshot );
    g_pcTimerLooper->Unlock();
    return( 0 );
}

void QTimer::startObjectTimer( int id, int msec, QObject *receiver )
{
	m_bIsObjectTimer = true;
	m_pcObject = receiver;
	m_bIsActive = true;
    m_bSingleShot = false;
    m_nObjectTimerID = id;
    g_pcTimerLooper->Lock();
    g_pcTimerLooper->AddTimer( this, id, bigtime_t(msec) * 1000LL, false );
    g_pcTimerLooper->Unlock();    
}

void QTimer::changeInterval( int msec )
{
    g_pcTimerLooper->Lock();
    if ( m_bIsActive ) {
	g_pcTimerLooper->RemoveTimer( this, ID_TIMER );
    }
    g_pcTimerLooper->AddTimer( this, ID_TIMER, bigtime_t(msec) * 1000LL, m_bSingleShot );
    m_bIsActive = true;
    g_pcTimerLooper->Unlock();
}

void QTimer::stop()
{
    g_pcTimerLooper->Lock();
    if ( m_bIsActive ) {
	g_pcTimerLooper->RemoveTimer( this, ID_TIMER );
	m_bIsActive = false;
    }
    g_pcTimerLooper->Unlock();
}

void QTimer::singleShot( int msec, QObject *receiver, const char *member )
{
    QTimer* pcTimer = new QTimer();
    receiver->connect( pcTimer, SIGNAL( timeout() ), receiver, member );
    pcTimer->start( msec, true );
    pcTimer->m_bDeleteWhenTrigged = true;    
}

int qStartTimer( int interval, QObject *obj )
{
	/* Find a free timer */
	g_pcTimerLooper->Lock();
	int i = 1;
	for( i = 1; i < TIMER_COUNT; i++ )
	{
		if( g_pcTimerList[i] == NULL )
		{
			//printf("Found timer slot %i %i\n", i, interval );
			/* Create timer */
			g_pcTimerList[i] = new QTimer();
			g_pcTimerList[i]->startObjectTimer( i, interval, obj );
			break;
		}
	}
	if( i == TIMER_COUNT )
	{
		printf("Error: No free timer slot found!\n" );
		g_pcTimerLooper->Unlock();		
		return( -1 );
	}
	g_pcTimerLooper->Unlock();
	return( i );
}

bool qKillTimer( int id )
{
	g_pcTimerLooper->Lock();
	if( g_pcTimerList[id] == NULL )
	{
		g_pcTimerLooper->Unlock();
		//printf("Could not find timer %i\n", id );
		return( false );
	}
	delete( g_pcTimerList[id] );
   	g_pcTimerList[id] = NULL;
	
	//printf("Timer %i deleted\n", id );
	g_pcTimerLooper->Unlock();
	return( true );
}

bool qKillTimer( QObject *obj )
{
	g_pcTimerLooper->Lock();
	for( int i = 1; i < TIMER_COUNT; i++ )
	{
		if( g_pcTimerList[i] != NULL )
		{
			if( g_pcTimerList[i]->GetObject() == obj )
			{
				printf("Found object timer %i\n", i );
				delete( g_pcTimerList[i] );				
				g_pcTimerList[i] = NULL;
			}
		}
	}
	g_pcTimerLooper->Unlock();
	return( true );
}

#include "qtimer.moc"

