
#include <qvariant.h>
#include <util/variant.h>

class QVariant::Private
{
public:
	QVariant::Type m_eType;
	bool m_bBool;
	int m_nInt;
	double m_fDouble;
	QString m_zString;
	os::Variant GetVariant()
	{
    	switch( m_eType )
    	{
    		case Bool:
    			return( os::Variant( m_bBool ) );
    		break;
    		case Int:
    			return( os::Variant( m_nInt ) );
    		break;
    		case Double:
    			return( os::Variant( m_fDouble ) );
    		break;
    		case String:
    			if( m_zString.utf8().isNull() )
    				return( os::Variant( "" ) );
    			else
	    			return( os::Variant( m_zString.utf8().data() ) );
	    	break;
    		default:
    			printf( "Error: QVariant::GetVariant(): invalid variant type!\n" );
    	}
    	return( os::Variant( 0 ) );
    }
};

QVariant::QVariant()
{
    d = new Private;
    d->m_eType = Invalid;
}


QVariant::QVariant( int nValue )
{
    d = new Private;
    d->m_eType = Int;
	d->m_nInt = nValue;
}

QVariant::QVariant( const QString& zValue )
{
    d = new Private;
    d->m_eType = String;
	d->m_zString = zValue;
}

QVariant::QVariant( bool bValue, int )
{
    d = new Private;
    d->m_eType = Bool;
	d->m_bBool = bValue;
}

QVariant::QVariant( double fValue )
{
    d = new Private;
    d->m_eType = QVariant::Double;
    d->m_fDouble = fValue;
}

QVariant::QVariant( const QVariant &cOther )
{
	d = new Private;
	d->m_eType = cOther.d->m_eType;
	d->m_bBool = cOther.d->m_bBool;
	d->m_fDouble = cOther.d->m_fDouble;
	d->m_zString = cOther.d->m_zString;
	d->m_nInt = cOther.d->m_nInt;
}

QVariant::~QVariant()
{
	delete( d );
}


QVariant& QVariant::operator=( const QVariant &cOther )
{
	d->m_eType = cOther.d->m_eType;
	d->m_bBool = cOther.d->m_bBool;
	d->m_fDouble = cOther.d->m_fDouble;
	d->m_zString = cOther.d->m_zString;
	d->m_nInt = cOther.d->m_nInt;
    return( *this );
}

bool QVariant::isValid() const
{
	return( d->m_eType != Invalid );
}

QVariant::Type QVariant::type() const
{
    return( d->m_eType );
}

bool QVariant::toBool() const
{
    return( d->GetVariant().AsBool() );
}

bool& QVariant::asBool() const
{
	d->m_bBool = d->GetVariant().AsBool();
	d->m_eType = Bool;
	return( d->m_bBool );
}


int QVariant::toInt() const
{
    return( d->GetVariant().AsInt32() );
}

int& QVariant::asInt() const
{
	d->m_nInt = d->GetVariant().AsInt32();
	d->m_eType = Int;
	return( d->m_nInt );
}

uint QVariant::toUInt() const
{
	return( d->GetVariant().AsInt32() );
}

QString QVariant::toString() const
{
    return( QString( d->GetVariant().AsString().c_str() ) );
}

QString& QVariant::asString() const
{
	d->m_zString = QString( d->GetVariant().AsString().c_str() );
	d->m_eType = String;
	return( d->m_zString );
}

QStringList QVariant::toStringList() const
{
	printf("QVariant::toStringList()\n" );
	QStringList cList( d->GetVariant().AsString().c_str() );
    return( cList );
}






