#include <khtml_part.h>
#include "../dropin/khtml/khtml_ext.h"
#include <khtmlview.h>
#include <kparts/factory.h>
#include <misc/loader.h>
#include <klibloader.h>
#include <ecma/kjs_proxy.h>
#include <kiconloader.h>
#include <qmutex.h>

#include "HTMLView.h"

using namespace os;

#if 0
extern "C"
{
	void* init_libkflash();
}
#endif

class HTMLPart : public KHTMLPart
{
public:
	HTMLPart() : KHTMLPart( NULL, "khtmlpart" )
	{
	}
	virtual KParts::ReadOnlyPart *createPart( QWidget *parentWidget, const char *widgetName,
                                            QObject *parent, const char *name,
                                            const QString &mimetype, QString &serviceName,
                                            QStringList &serviceTypes, const QStringList &params)
	{
		#if 0
		printf("Create new part %s %s %s\n", name, mimetype.utf8().data(), serviceName.utf8().data() );
		if( mimetype == "application/x-shockwave-flash" || mimetype == "application/x-futuresplash" )
		{
			printf("Create flash part\n" );
			KParts::Factory* pcFactory = (KParts::Factory*)init_libkflash();
			return(  static_cast<KParts::ReadOnlyPart *>(pcFactory->createPart( parentWidget, widgetName, parent, name,
	    serviceName.latin1(), params ) ) );
		}
		#endif
		return( KHTMLPart::createPart( parentWidget, widgetName, parent, name, mimetype, serviceName,
				serviceTypes, params ) );
	}
};

class HTMLView::Private
{
public:
	HTMLPart * m_pcPart;
	int m_nCurHistoryPos;
	std::vector < HistoryEntry * >m_cHistory;
};

HTMLView::HTMLView( const Rect & cFrame, const String &cTitle, uint32 nResizeMask, uint32 nFlags ):os::View( cFrame, cTitle, nResizeMask, nFlags )
{
	m = new Private;
	SetPart( new HTMLPart() );
	GetPart()->view()->SetTopLevelWidget( true );
	AddChild( GetPart()->view(  ) );
	m->m_nCurHistoryPos = 0;
}

HTMLView::~HTMLView()
{
	delete( m->m_pcPart );
	delete( m );
}

void HTMLView::KeyDown( const char *pzString, const char *pzRawString, uint32 nQualifiers )
{
	if ( nQualifiers & QUAL_CTRL )
	{
		switch ( pzRawString[0] )
		{
		case VK_TAB:
			GetWindow()->PostMessage( ID_ACTIVATE_URLEDIT, GetWindow(  ) );
			break;
		case 'r':
			GetWindow()->PostMessage( ID_RELOAD, GetWindow(  ) );
			break;
		case 'f':
			GetWindow()->PostMessage( ID_FIND, GetWindow(  ) );
			break;
		case VK_INSERT:
		case 'c':
			GetWindow()->PostMessage( ID_COPY, GetWindow(  ) );
			break;
		case 'n':
			GetWindow()->PostMessage( ID_NEW_WINDOW, GetWindow(  ) );
			break;
		default:
			View::KeyDown( pzString, pzRawString, nQualifiers );
			break;
		}
	}
	else
	{
		switch ( pzRawString[0] )
		{
		case VK_BACKSPACE:
			GetWindow()->PostMessage( ID_PREV_URL, GetWindow(  ) );
			break;
		case VK_FUNCTION_KEY:
			{
				os::Message * pcMsg = GetLooper()->GetCurrentMessage(  );
				assert( pcMsg != NULL );
				int32 nKeyCode;

				if ( pcMsg->FindInt32( "_raw_key", &nKeyCode ) != 0 )
				{
					return;
				}

				switch ( nKeyCode )
				{
				case 4:	// F3
					GetWindow()->PostMessage( ID_REPEAT_FIND, GetWindow(  ) );
					break;
				default:
					View::KeyDown( pzString, pzRawString, nQualifiers );
					break;
				}
				break;
			}
		default:
			View::KeyDown( pzString, pzRawString, nQualifiers );
			break;

		}
	}
}

void HTMLView::SetPart( HTMLPart * pcPart )
{
	m->m_pcPart = pcPart;
}


HistoryEntry *HTMLView::GetCurrentHistory()
{
	if ( m->m_nCurHistoryPos >= 0 && m->m_nCurHistoryPos < int ( m->m_cHistory.size() ) )
	{
		return ( m->m_cHistory[m->m_nCurHistoryPos] );
	}
	else
	{
		return ( NULL );
	}
}

KHTMLPart *HTMLView::GetPart()
{
	return ( m->m_pcPart );
}

int HTMLView::GetHistoryPosition()
{
	return ( m->m_nCurHistoryPos );
}

void HTMLView::SetHistoryPosition( int nPos )
{
	m->m_nCurHistoryPos = nPos;
}

std::vector < HistoryEntry * >HTMLView::GetHistoryEntries()
{
	return ( m->m_cHistory );
}


os::String HTMLView::GetCurrentURL()
{
	return ( os::String ( GetPart()->url(  ).prettyURL(  ).utf8(  ).data(  ) ) );
}

void HTMLView::OpenURL( const os::String cURL, const KParts::URLArgs & cArgs )
{
	m->m_pcPart->browserExtension()->setURLArgs( cArgs );
	m->m_pcPart->openURL( cURL.c_str() );
}

void HTMLView::AddHistoryEntry()
{
	m->m_nCurHistoryPos++;
	for ( uint i = m->m_nCurHistoryPos; i < m->m_cHistory.size(); ++i )
	{
		delete m->m_cHistory[i];
	}
	if ( m->m_nCurHistoryPos > 0 )
	{
		m->m_cHistory.resize( m->m_nCurHistoryPos );
	}
	HistoryEntry *pcEntry = new HistoryEntry;
	pcEntry->m_cURL = "";
	m->m_cHistory.push_back( pcEntry );
}

bool HTMLView::GoHistory( int nStep )
{
	m->m_nCurHistoryPos += nStep;
	HistoryEntry *pcEntry = GetCurrentHistory();

	if ( pcEntry )
	{
		GetPart()->browserExtension(  )->restoreState( pcEntry->m_cState );
		return ( false );
	}
	return ( false );
}

void HTMLView::UpdateHistory()
{
	if ( GetPart()->url(  ).prettyURL(  ).utf8(  ).data(  ) == NULL )
		return;
	
	HistoryEntry *pcEntry = GetCurrentHistory();

	if ( pcEntry )
	{
		pcEntry->m_cState.MakeEmpty();
	}
	else
	{
		pcEntry = new HistoryEntry;
		m->m_nCurHistoryPos = m->m_cHistory.size();
		m->m_cHistory.push_back( pcEntry );
	}
	
	GetPart()->browserExtension(  )->saveState( &pcEntry->m_cState );
	pcEntry->m_cURL = GetPart()->url(  );
}






