#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream>

#include <atheos/time.h>

#include <util/application.h>
#include <util/message.h>
#include <util/messenger.h>
#include <util/clipboard.h>
#include <util/resources.h>
#include <util/settings.h>
#include <gui/window.h>
#include <gui/view.h>
#include <gui/frameview.h>
#include <gui/button.h>
#include <gui/textview.h>
#include <gui/tabview.h>

#include <khtml_part.h>
#include <khtmlview.h>
#include <khtml_factory.h>
#include <misc/loader.h>
#include <kapplication.h>
#include <klibloader.h>
#include <ecma/kjs_proxy.h>
#include <kiconloader.h>
#include <css/cssstyleselector.h>
#include <rendering/render_style.h>

#include <string>
#include <list>

#include <gui/menu.h>
#include <gui/requesters.h>
#include <storage/path.h>
#include <storage/file.h>
#include <storage/directory.h>
#include <storage/tempfile.h>
#include <storage/registrar.h>

#include <qmutex.h>

#include "toolbar.h"
#include "statusbar.h"
#include "searchdialog.h"
#include "downloaddialog.h"

#include "bookmarks.h"
#include "main.h"

#include "HTMLView.h"
#include "../kio/http/kcookiejar/kcookiejar.h"
extern KCookieJar g_cCookieJar;


static KApplication *g_pcKApp;

using namespace os;

struct DownloadNode
{
	DownloadNode()
	{
		m_psDownloadStream = NULL;
		m_pcRequester = NULL;
		m_pcProgressDlg = NULL;
		m_nContentSize = -1;
		m_pcTmpFile = NULL;
		m_pcFile = NULL;
		m_nBytesReceived = 0;
		m_bDone = false;
		m_bCanceled = false;
	}

	KParts::DownloadStream * m_psDownloadStream;
	DownloadRequester *m_pcRequester;
	DownloadProgress *m_pcProgressDlg;

	std::string m_cURL;
	std::string m_cPreferredName;
	off_t m_nContentSize;
	bigtime_t m_nStartTime;

	std::string m_cPath;
	os::TempFile * m_pcTmpFile;
	os::File * m_pcFile;
	int m_nBytesReceived;
	bool m_bDone;
	bool m_bCanceled;
};


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

std::vector<BrowserWindow*> BrowserWindow::s_cWindows;

TextView *BrowserWindow::GetURLView()
{
	return ( m_pcURLView );
}

KHTMLPart *BrowserWindow::GetPart()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	return ( hview->GetPart() );
}

void BrowserWindow::SwitchTab()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	m_pcURLView->Set( hview->GetCurrentURL().c_str(  ), false );
	SetTitle( m_pcTabs->GetTabTitle( i ) + " - ABrowse" );
	std::string cURL = m_pcURLView->GetBuffer()[0];

	KHTMLView *pcView = hview->GetPart()->view(  );

	pcView->SetResizeMask( CF_FOLLOW_ALL );

	os::font_height sFontHeight;
	m_pcTabs->GetFontHeight( &sFontHeight );
	float vGlyphHeight = sFontHeight.ascender + sFontHeight.descender + sFontHeight.line_gap;
	float vTabHeight = ceil( vGlyphHeight * 1.2f + 6.0f );

	os::Rect cClientBounds = m_pcTabs->GetBounds();
	cClientBounds.Resize( 0.0f, 0.0f, -4.0f, -2.0f - vTabHeight );
	cClientBounds.Floor();

	pcView->SetFrame( cClientBounds );

	pcView->MakeFocus();

//	pcView->SetMsgTarget( Messenger( this ) );

	
	GoHistory( 0, true );

}

os::Point BrowserWindow::GetNewWinPosition()
{
	static Point cLastPos( 100, 100 );

	cLastPos.x += 30;
	cLastPos.y += 30;
	if ( cLastPos.x > 500 )
	{
		cLastPos.x = 100;
	}
	if ( cLastPos.y > 500 )
	{
		cLastPos.y = 100;
	}
	return ( cLastPos );
}

void BrowserWindow::CreateNewTab()
{
	HTMLView *hview = new HTMLView( m_pcTabs->GetBounds(), "html_tab", os::CF_FOLLOW_ALL );

	hview->GetPart()->browserExtension(  )->SetCallback( this );
	hview->GetPart()->view()->show();
	m_pcTabs->AppendTab( "  (Untitled)  ", hview );

}
KParts::ReadOnlyPart* BrowserWindow::CreateNewTab( const std::string & cURL, const KParts::URLArgs & cArgs )
{
	HTMLView *hview = new HTMLView( m_pcTabs->GetBounds(), "html_tab", os::CF_FOLLOW_ALL );

	hview->GetPart()->browserExtension(  )->SetCallback( this );
	KHTMLView *pcView = hview->GetPart()->view(  );
	pcView->show();
	m_pcTabs->AppendTab( "  (Untitled)  ", hview );
	hview->OpenURL( cURL, cArgs );

	return( hview->GetPart() );
}

void BrowserWindow::DeleteTab()
{
	uint i = m_pcTabs->GetSelection();
	int nt = m_pcTabs->GetTabCount();

	if ( nt > 1 )
	{
		delete( m_pcTabs->DeleteTab( i ) );
	}
}

BrowserWindow::BrowserWindow( const Rect & cFrame, bool bLoadPos ):Window( cFrame, "abrowse_wnd", "ABrowse", 0 )
{
	m_bSaveSize = bLoadPos;
	m_pcSearchDialog = NULL;
	m_bSearchCaseSensitive = false;
	char *pzHome = getenv( "HOME" );

	s_cWindows.push_back( this );

	GlobalMutex::Lock();
	SetMutex( GlobalMutex::GetMutex() );

	m_pcFrame = new LayoutView( GetBounds(), "frame_view" );

	m_pcTabs = new os::TabView( Rect( 0, 0, 0, 0 ), "tabs", CF_FOLLOW_LEFT | CF_FOLLOW_TOP, WID_WILL_DRAW | WID_CLEAR_BACKGROUND );
	m_pcTabs->SetMessage( new Message( ID_TAB_CHANGED ) );

	VLayoutNode *pcRoot = new VLayoutNode( "root" );
	m_pcMenuBar = new Menu( Rect( 0, 0, 0, 0 ), "menu_bar", ITEMS_IN_ROW );
	Menu *pcAppMenu = new Menu( Rect( 0, 0, 0, 0 ), "Application", ITEMS_IN_COLUMN );

	pcAppMenu->AddItem( "About", new os::Message( ID_ABOUT ) );
	pcAppMenu->AddItem( new os::MenuSeparator() );
	pcAppMenu->AddItem( "Quit", new os::Message( M_QUIT ) );
	m_pcMenuBar->AddItem( pcAppMenu );

	Menu *pcTabMenu = new Menu( Rect( 0, 0, 0, 0 ), "Tab", ITEMS_IN_COLUMN );

	pcTabMenu->AddItem( "New Tab", new os::Message( ID_NEW_TAB ) );
	pcTabMenu->AddItem( "Close Tab", new os::Message( ID_DELETE_TAB ) );

	m_pcMenuBar->AddItem( pcTabMenu );
	Menu *pcEditMenu = new Menu( Rect( 0, 0, 0, 0 ), "Edit", ITEMS_IN_COLUMN );

	pcEditMenu->AddItem( "Copy", new os::Message( ID_COPY ) );
	pcEditMenu->AddItem( "Paste", new os::Message( ID_PASTE ) );
	m_pcMenuBar->AddItem( pcEditMenu );
	

	m_pcBookmarkMenu = new BookmarksMenu( this, os::Path( os::String ( pzHome ) + "/Bookmarks" ), os::Rect(), "Bookmarks", os::ITEMS_IN_COLUMN );

	m_pcBookmarkMenu->SetTargetForItems( this );
	m_pcMenuBar->AddItem( m_pcBookmarkMenu );
	
	m_pcMenuBar->SetTargetForItems( this );

	m_pcURLView = new TextView( Rect( 0, 0, 0, 0 ), "url_view", "" /*g_pzDefaultURL */  );

	m_pcURLView->SetEventMask( os::TextView::EI_ENTER_PRESSED | os::TextView::EI_ESC_PRESSED );
	m_pcURLView->SetMultiLine( false );
	m_pcURLView->SetMessage( new Message( ID_URL_CHANGED ) );

	m_pcToolBar = new ToolBar( Rect( 0, 0, 0, 0 ), "tool_bar" );
	m_pcToolBar->AddButton( "breaker.png", "", NULL );
	m_pcToolBar->AddButton( "back.png", "Back", new os::Message( ID_PREV_URL ) );
	m_pcToolBar->AddButton( "forward.png", "Forward", new os::Message( ID_NEXT_URL ) );
	m_pcToolBar->AddButton( "home.png", "Home", new os::Message( ID_GOHOME ) );
	m_pcToolBar->AddButton( "reload.png", "Reload", new os::Message( ID_RELOAD ) );
	m_pcToolBar->AddButton( "find.png", "Find", new os::Message( ID_FIND ) );
	m_pcToolBar->GetNode()->AddChild( m_pcURLView );
	m_pcToolBar->Finish();

	m_pcStatusBar = new StatusBar( Rect(), "status_bar" );

	pcRoot->AddChild( m_pcMenuBar );
	pcRoot->AddChild( m_pcToolBar, 0.0f );

	CreateNewTab();

	pcRoot->AddChild( m_pcTabs );
	pcRoot->AddChild( m_pcStatusBar );
	m_pcFrame->SetRoot( pcRoot );

	AddChild( m_pcFrame );




	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
	KHTMLView *pcView = hview->GetPart()->view(  );


	pcView->SetResizeMask( CF_FOLLOW_ALL );

	os::font_height sFontHeight;
	m_pcTabs->GetFontHeight( &sFontHeight );
	float vGlyphHeight = sFontHeight.ascender + sFontHeight.descender + sFontHeight.line_gap;
	float vTabHeight = ceil( vGlyphHeight * 1.2f + 6.0f );

	os::Rect cClientBounds = m_pcTabs->GetBounds();
	cClientBounds.Resize( 0.0f, 0.0f, -4.0f, -2.0f - vTabHeight );
	cClientBounds.Floor();
	pcView->SetFrame( cClientBounds );

	pcView->MakeFocus();

//	pcView->SetMsgTarget( Messenger( this ) );

	pcView->show();
	hview->GetPart()->browserExtension(  )->SetCallback( this );
	m_pcURLView->SetTarget( this );

	m_pcToolBar->EnableButton( ID_PREV_URL, false );
	m_pcToolBar->EnableButton( ID_NEXT_URL, false );
	m_pcToolBar->EnableButton( ID_COPY, false );

	if ( bLoadPos && NULL != pzHome )
	{
		/* Load settings */
		os::Settings * pcSettings = new os::Settings();
		if ( pcSettings->Load() == 0 )
		{
			os::Rect cNewFrame = pcSettings->GetRect( "frame", GetFrame() );
			SetFrame( cNewFrame );
		} else
			CenterInScreen();
		delete( pcSettings );
	}

	/* Set Icon */
	os::Resources cCol( get_image_id() );
	os::ResStream * pcStream = cCol.GetResourceStream( "icon24x24.png" );
	os::BitmapImage * pcIcon = new os::BitmapImage( pcStream );
	delete( pcStream );
	SetIcon( pcIcon->LockBitmap() );
	delete( pcIcon );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

BrowserWindow::~BrowserWindow()
{
	/* Stop loading */
	for ( int i = 0; i < m_pcTabs->GetTabCount(); ++i )
	{
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

		hview->GetPart()->closeURL();
	}
	RemoveChild( m_pcFrame );
	delete( m_pcFrame );
	/* Delete window from list */
	Lock();
	for( uint i = 0; i < s_cWindows.size(); i++ ) {
		if( s_cWindows[i] == this )
		{
			s_cWindows.erase( s_cWindows.begin() + i );
			break;
		}
	}
	Unlock();
}


void BrowserWindow::UpdateHistory()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	hview->UpdateHistory();
}


void BrowserWindow::AddHistoryEntry()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	hview->AddHistoryEntry();
	m_pcToolBar->EnableButton( ID_NEXT_URL, false );
	m_pcToolBar->EnableButton( ID_PREV_URL, true );
}

void BrowserWindow::GoHistory( int nStep, bool bButtonsOnly )
{

	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	if ( !bButtonsOnly )
	{
		hview->GoHistory( nStep );
	}
	HistoryEntry *pcCurrent = hview->GetCurrentHistory();

	if ( pcCurrent != NULL )
	{
		m_pcURLView->Set( pcCurrent->m_cURL.prettyURL().utf8(  ).data(  ), false );
	}
	if ( hview->GetHistoryPosition() == 0 )
	{
		m_pcToolBar->EnableButton( ID_PREV_URL, false );
	}
	else
	{
		m_pcToolBar->EnableButton( ID_PREV_URL, true );
	}
	if ( hview->GetHistoryPosition() >= int ( hview->GetHistoryEntries(  ).size(  ) ) - 1 )
	{
		m_pcToolBar->EnableButton( ID_NEXT_URL, false );
	}
	else
	{
		m_pcToolBar->EnableButton( ID_NEXT_URL, true );
	}
}

#if 0
void dump_message( os::Message * pcMsg )
{
	int nCount = pcMsg->CountNames();

	for ( int i = 0; i < nCount; ++i )
	{
		int nType;
		int nCount;

		pcMsg->GetNameInfo( pcMsg->GetName( i ).c_str(), &nType, &nCount );
		switch ( nType )
		{
		case T_STRING:
			{
				std::string cStr;
				pcMsg->FindString( pcMsg->GetName( i ).c_str(), &cStr );
				printf( "%s: '%s'\n", pcMsg->GetName( i ).c_str(), cStr.c_str(  ) );
				break;
			}
		default:
			printf( "%s: (Type=%d)\n", pcMsg->GetName( i ).c_str(), nType );
			break;

		}
	}
}
#endif

void BrowserWindow::OpenURL( const std::string & cURL, const KParts::URLArgs & cArgs )
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
	os::String cURLSet = cURL;
	m_pcURLView->Set( cURLSet.c_str(), false );
	if ( cURL.find( "#" ) == std::string::npos )
		m_pcTabs->SetTabTitle( i, "Loading..." );
	hview->OpenURL( cURL, cArgs );
}

void BrowserWindow::OpenURLRequest( const std::string & cURL, const KParts::URLArgs & cURLArgs )
{
	UpdateHistory();
	AddHistoryEntry();
	OpenURL( cURL, cURLArgs );
	UpdateHistory();
}

void BrowserWindow::OpenURLNotify()
{
	UpdateHistory();
	AddHistoryEntry();
}

void BrowserWindow::Completed()
{
	UpdateHistory();
}

void BrowserWindow::SetLocationBarURL( const std::string & cURL )
{
	m_pcURLView->Set( cURL.c_str(), false );
}

void BrowserWindow::CreateNewWindow( const std::string & cURL, const KParts::URLArgs & cURLArgs )
{
	os::Rect cFrame( 0, 0, 799, 599 );

	cFrame += GetNewWinPosition();

	BrowserWindow *pcNewWindow = new BrowserWindow( cFrame, false );

	if ( cURL.empty() == false )
	{
		pcNewWindow->AddHistoryEntry();
		pcNewWindow->OpenURL( cURL, cURLArgs );
	}
	pcNewWindow->Show( true );
	pcNewWindow->MakeFocus( true );
}

KParts::ReadOnlyPart * BrowserWindow::CreateNewWindow( const std::string & cURL, const KParts::URLArgs & cURLArgs, const KParts::WindowArgs & sWndArgs )
{
	if( sWndArgs.lowerWindow )
	{
		return( CreateNewTab( cURL, cURLArgs ) );
	}
	
	os::Rect cFrame( 0, 0, 799, 599 );

	Point cPos;

	if ( sWndArgs.x == -1 || sWndArgs.y == -1 )
	{
		cPos = GetNewWinPosition();
	}
	if ( sWndArgs.x != -1 )
	{
		cPos.x = sWndArgs.x;
	}
	if ( sWndArgs.y != -1 )
	{
		cPos.y = sWndArgs.y;
	}
	if ( sWndArgs.width != -1 )
	{
		cFrame.right = cFrame.left + sWndArgs.width - 1;
	}
	if ( sWndArgs.height != -1 )
	{
		cFrame.bottom = cFrame.top + sWndArgs.height - 1;
	}
	cFrame += cPos + os::Point( 20, 20 );

//    std::vector<HistoryEntry*> cHistory = m_cHistory;
	//  cHistory.resize( m_nCurHistoryPos + 1 );
	BrowserWindow *pcNewWindow = new BrowserWindow( cFrame, false );

	if ( cURL.empty() == false )
	{
		pcNewWindow->AddHistoryEntry();
		pcNewWindow->OpenURL( cURL, cURLArgs );
	}
	pcNewWindow->Show( true );
	pcNewWindow->MakeFocus( true );
	return ( pcNewWindow->GetPart() );

}

void BrowserWindow::PopupMenu( KParts::BrowserExtension * /*pcSource */ , const std::string & cURL )
{
	m_pcPopupMenu = new os::Menu( Rect( 0, 0, 10, 10 ), "popup", ITEMS_IN_COLUMN );
	m_pcPopupMenu->SetCloseMessage( os::Message( ID_POPUP_CLOSED ) );
	m_pcPopupMenu->SetCloseMsgTarget( this );

	os::Message * pcMsg;
	m_pcPopupMenu->AddItem( new os::MenuItem( "New window", new os::Message( ID_NEW_WINDOW ) ) );
	m_pcPopupMenu->AddItem( new os::MenuItem( "Reload", new os::Message( ID_RELOAD ) ) );

	if ( cURL.empty() == false )
	{
		m_pcPopupMenu->AddItem( new os::MenuSeparator() );

		pcMsg = new os::Message( ID_OPEN_TAB );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Open link in new tab", pcMsg ) );

		pcMsg = new os::Message( ID_OPEN_LINK );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Open link in new window", pcMsg ) );

		pcMsg = new os::Message( ID_SAVE_LINK );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Save link as...", pcMsg ) );

		pcMsg = new os::Message( ID_COPY_LINK_LOCATION );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Copy link location", pcMsg ) );
	}

	m_pcPopupMenu->AddItem( new os::MenuSeparator() );
	m_pcPopupMenu->AddItem( new os::MenuItem( "Quit", new os::Message( M_QUIT ) ) );

	m_pcPopupMenu->SetTargetForItems( this );

	os::Point cMousePos;
	m_pcStatusBar->GetMouse( &cMousePos, NULL );
	m_pcPopupMenu->Open( m_pcStatusBar->ConvertToScreen( cMousePos ) );
}

void BrowserWindow::LoadingProgress( int nPercent )
{
	m_pcStatusBar->SetProgress( nPercent );
}
void BrowserWindow::SpeedProgress( int /*bytesPerSecond */  )
{
}
void BrowserWindow::InfoMessage( const std::string & cString )
{
	m_pcStatusBar->SetStatus( cString );
}

void BrowserWindow::SetWindowCaption( KParts::ReadOnlyPart * pcPart, const std::string & cCaption )
{
	Lock();

	for ( int i = 0; i < m_pcTabs->GetTabCount(); ++i )
	{
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

		if ( pcPart == hview->GetPart() )
		{
			if ( ( int )m_pcTabs->GetSelection() == i )
				SetTitle( cCaption + " - ABrowse" );
			m_pcTabs->SetTabTitle( i, cCaption );
		}
	}

	Unlock();
}

void BrowserWindow::SetStatusBarText( const std::string & cString )
{
	m_pcStatusBar->SetStatus( cString );
}


void BrowserWindow::SelectionInfo( const std::string & cText )
{
	m_cSelectedText = cText;
	m_pcToolBar->EnableButton( ID_COPY, m_cSelectedText.empty() == false );
}

bool BrowserWindow::BeginDownload( KParts::DownloadStream * psStream )
{
	DownloadNode *psNode = new DownloadNode;

	psNode->m_nStartTime = get_system_time();
	psNode->m_cURL = psStream->m_cURL;
	psNode->m_cPreferredName = psStream->m_cPreferredName;
	psNode->m_nContentSize = psStream->m_nContentSize;

	try
	{
		psNode->m_pcTmpFile = new os::TempFile( "ab-", "/tmp" );
	} catch( ... )
	{
		printf( "Error: BrowserWindow::BeginDownload() failed to create temporary file\n" );
		return ( false );
	}

	Message cOkMsg( ID_START_DOWNLOAD );
	Message cCancelMsg( ID_CANCEL_DOWNLOAD );

	cOkMsg.AddPointer( "node", psNode );
	cCancelMsg.AddPointer( "node", psNode );

	psNode->m_psDownloadStream = psStream;
	psNode->m_pcRequester = new DownloadRequester( Rect( 250, 200, 599, 300 ), "File Download", psStream->m_cURL, psStream->m_cPreferredName, psStream->m_cMimeType, psStream->m_nContentSize, Messenger( this ), cOkMsg, cCancelMsg );
	psNode->m_pcRequester->Show();
	psNode->m_pcRequester->MakeFocus();
	psStream->m_pUserData = psNode;
	return ( true );
}

bool BrowserWindow::HandleDownloadData( KParts::DownloadStream * psStream, const void *pData, int nLen )
{
	DownloadNode *psNode = static_cast < DownloadNode * >( psStream->m_pUserData );

	if ( psNode == NULL || psNode->m_bCanceled || ( psNode->m_pcTmpFile == NULL && psNode->m_pcFile == NULL ) )
	{
		return ( false );
	}
	int nBytesWritten;

	if ( psNode->m_pcFile != NULL )
	{
		nBytesWritten = psNode->m_pcFile->Write( pData, nLen );
	}
	else
	{
		nBytesWritten = psNode->m_pcTmpFile->Write( pData, nLen );
	}
	if ( nBytesWritten != nLen )
	{
		( new os::Alert( "Error: Failed to create file!", os::String ().Format( "Failed to write %df bytes to\n%s\n\nError: %s\n", psNode->m_cPath.c_str(  ), strerror( errno ) ), 0, "Sorry!", NULL ) )->Go( NULL );

		psNode->m_bCanceled = true;

		if ( psNode->m_pcProgressDlg != NULL )
		{
			psNode->m_pcProgressDlg->Terminate();
			psNode->m_pcProgressDlg = NULL;
		}
		if ( psNode->m_pcRequester == NULL )
		{
			delete psNode->m_pcTmpFile;
			delete psNode->m_pcFile;

			if ( psNode->m_cPath.empty() == false )
			{
				unlink( psNode->m_cPath.c_str() );
			}
			delete psNode;
		}
		else
		{
			psNode->m_bDone = true;
		}

	}
	else
	{
		psNode->m_nBytesReceived += nLen;
		if ( psNode->m_pcProgressDlg != NULL )
		{
			psNode->m_pcProgressDlg->UpdateProgress( psNode->m_nBytesReceived );
		}
	}
	return ( true );
}

void BrowserWindow::DownloadFinished( KParts::DownloadStream * psStream, int /* nError */  )
{
	DownloadNode *psNode = static_cast < DownloadNode * >( psStream->m_pUserData );

	if ( psNode == NULL || psNode->m_bCanceled || ( psNode->m_pcTmpFile == NULL && psNode->m_pcFile == NULL ) )
	{
		return;
	}

	if ( psNode->m_pcProgressDlg != NULL )
	{
		psNode->m_pcProgressDlg->Terminate();
		psNode->m_pcProgressDlg = NULL;
	}
	if ( psNode->m_pcRequester == NULL )
	{
		delete psNode->m_pcTmpFile;
		delete psNode->m_pcFile;
		delete psNode;
	}
	else
	{
		psNode->m_bDone = true;
	}
}

void BrowserWindow::HandleMessage( Message * pcMsg )
{
	switch ( pcMsg->GetCode() )
	{
	case ID_TAB_CHANGED:
		{
			SwitchTab();
			break;
		}
	case ID_NEW_TAB:
		{
			CreateNewTab();
			break;
		}
	case ID_DELETE_TAB:
		{
			DeleteTab();
			break;
		}
	case ID_URL_CHANGED:
		{
			uint32 nEvents = 0;

			pcMsg->FindInt( "events", &nEvents );
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( nEvents & os::TextView::EI_ESC_PRESSED )
			{
				m_pcURLView->Set( hview->GetCurrentURL().c_str(  ) );
				hview->GetPart()->view(  )->MakeFocus(  );
				break;
			}
			if ( nEvents & os::TextView::EI_ENTER_PRESSED )
			{
				std::string cURL = m_pcURLView->GetBuffer()[0];

				for ( uint i = 0; i <= cURL.size(); ++i )
				{
					if ( i == cURL.size() || isalnum( cURL[i] ) == false )
					{
						if ( i == cURL.size() || cURL[i] != ':' )
						{
							std::string cTmp = cURL;
							cURL = "http://";
							for ( uint j = 0; j < cTmp.size(); ++j )
							{
								if ( cTmp[j] != '/' )
								{
									cURL.insert( cURL.end(), cTmp.begin(  ) + j, cTmp.end(  ) );
									break;
								}
							}
						}
						break;
					}
				}

				hview->GetPart()->view(  )->MakeFocus(  );
				OpenURLRequest( cURL, KParts::URLArgs() );

			}
			break;
		}
	case ID_ACTIVATE_URLEDIT:
		m_pcURLView->MakeFocus( true );
		m_pcURLView->SetCursor( -1, 0 );
		m_pcURLView->SelectAll();
		break;
	case ID_RELOAD:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			KParts::URLArgs cArgs;
			cArgs.reload = true;
			cArgs.xOffset = hview->GetPart()->browserExtension(  )->xOffset(  );
			cArgs.yOffset = hview->GetPart()->browserExtension(  )->yOffset(  );
			OpenURL( m_pcURLView->GetBuffer()[0], cArgs );
			break;
		}
	case ID_NEW_WINDOW:
		{
			UpdateHistory();
			BrowserWindow *pcNewWindow = new BrowserWindow( GetFrame() + os::Point( 30.0f, 30.0f ), false );

			pcNewWindow->GoHistory( 0, false );
			pcNewWindow->Show( true );
			pcNewWindow->MakeFocus( true );
			break;
		}
	case ID_OPEN_TAB:
		{
			std::string cURL;
			pcMsg->FindString( "url", &cURL );
			CreateNewTab( cURL, KParts::URLArgs() );
			break;
		}
	case ID_OPEN_LINK:
		{
			std::string cURL;
			pcMsg->FindString( "url", &cURL );
			BrowserWindow *pcNewWindow = new BrowserWindow( GetFrame() + os::Point( 30.0f, 30.0f ), false );

			pcNewWindow->AddHistoryEntry();
			pcNewWindow->OpenURL( cURL, KParts::URLArgs() );
			pcNewWindow->Show( true );
			pcNewWindow->MakeFocus( true );
			break;
		}
	case ID_SAVE_LINK:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			std::string cURL;
			pcMsg->FindString( "url", &cURL );

			hview->GetPart()->browserExtension(  )->SaveURL( cURL, "Save link as..." );
			break;
		}
	case ID_COPY_LINK_LOCATION:
		{
			std::string cURL;
			pcMsg->FindString( "url", &cURL );

			os::Clipboard cClipboard;
			cClipboard.Lock();
			cClipboard.Clear();
			os::Message * pcData = cClipboard.GetData();
			pcData->AddString( "text/plain", cURL );
			cClipboard.Commit();
			cClipboard.Unlock();
			break;
		}
	case ID_PASTE:
		{
			GetURLView()->Delete(  );
			GetURLView()->Paste( false );

			break;
		}
	case ID_COPY:
		{
			if ( m_cSelectedText.empty() )
			{
				break;
			}
			os::Clipboard cClipboard;
			cClipboard.Lock();
			cClipboard.Clear();
			os::Message * pcData = cClipboard.GetData();
			pcData->AddString( "text/plain", m_cSelectedText );
			cClipboard.Commit();
			cClipboard.Unlock();
			break;
		}
	case ID_FIND:
		if ( m_pcSearchDialog == NULL )
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			hview->GetPart()->findTextBegin(  );
			m_pcSearchDialog = new FindDialog( os::Messenger( this ), os::Message( ID_DO_SEARCH ), os::Message( ID_SEARCH_CLOSED ) );
			m_pcSearchDialog->Show();
			m_pcSearchDialog->MakeFocus();
		}
		break;
	case ID_DO_SEARCH:
		{
			bool bFromTop = false;

			pcMsg->FindBool( "from_top", &bFromTop );
			pcMsg->FindBool( "case_sensitive", &m_bSearchCaseSensitive );
			pcMsg->FindString( "string", &m_cLastSearchStr );

			KHTMLView *pcActiveView = dynamic_cast < KHTMLView * >( GetFocusChild() );
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
			KHTMLPart *pcPart = hview->GetPart();

			if ( pcActiveView != NULL )
			{
				pcPart = pcActiveView->part();
			}
			if ( bFromTop )
			{
				pcPart->findTextBegin();
			}
			pcPart->findTextNext( m_cLastSearchStr.c_str(), true, m_bSearchCaseSensitive, false );
			break;
		}
	case ID_REPEAT_FIND:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( m_cLastSearchStr.empty() == false )
			{
				KHTMLView *pcActiveView = dynamic_cast < KHTMLView * >( GetFocusChild() );

				KHTMLPart *pcPart = hview->GetPart();

				if ( pcActiveView != NULL )
				{
					pcPart = pcActiveView->part();
				}
				pcPart->findTextNext( m_cLastSearchStr.c_str(), true, m_bSearchCaseSensitive, true );
			}
			else if ( m_pcSearchDialog == NULL )
			{
				hview->GetPart()->findTextBegin(  );
				m_pcSearchDialog = new FindDialog( os::Messenger( this ), os::Message( ID_DO_SEARCH ), os::Message( ID_SEARCH_CLOSED ) );
				m_pcSearchDialog->Show();
				m_pcSearchDialog->MakeFocus();
			}
			break;
		}
	case ID_SEARCH_CLOSED:
		m_pcSearchDialog = NULL;
		break;
	case ID_GOHOME:
		UpdateHistory();
		AddHistoryEntry();
		OpenURL( "http://syllable.sourceforge.net/", KParts::URLArgs() );
		UpdateHistory();
		break;
	case ID_PREV_URL:
		GoHistory( -1, false );
		break;
	case ID_NEXT_URL:
		GoHistory( 1, false );
		break;
	case ID_START_DOWNLOAD:
		{
			DownloadNode *psNode = NULL;

			std::string cPath;
			pcMsg->FindPointer( "node", ( void ** )&psNode );
			pcMsg->FindString( "path", &cPath );


			if ( psNode == NULL )
			{
				break;	// Should never happen
			}
			psNode->m_cPath = cPath;
			const std::string cDstDir = os::Path( cPath.c_str() ).GetDir(  ).GetPath(  ).c_str(  );

			try
			{
				os::FSNode cDirNode( cDstDir );
				if ( cDirNode.GetDev() == psNode->m_pcTmpFile->GetDev(  ) )
				{
					rename( psNode->m_pcTmpFile->GetPath().c_str(  ), cPath.c_str(  ) );
					psNode->m_pcTmpFile->Detatch();	// Make sure the destructor don't do anything silly.
				}
				else
				{
					psNode->m_pcFile = new os::File( cPath, O_WRONLY | O_CREAT );
					psNode->m_pcTmpFile->Seek( 0, SEEK_SET );
					char anBuffer[32 * 1024];

					for ( ;; )
					{
						int nLen = psNode->m_pcTmpFile->Read( anBuffer, sizeof( anBuffer ) );

						if ( nLen < 0 )
						{
							throw os::errno_exception( "Failed to copy temporary file" );
						}
						if ( psNode->m_pcFile->Write( anBuffer, nLen ) != nLen )
						{
							throw os::errno_exception( "Failed to copy temporary file" );
						}
						if ( nLen < int ( sizeof( anBuffer ) ) )
						{
							break;
						}
					}
					delete psNode->m_pcTmpFile;

					psNode->m_pcTmpFile = NULL;
				}
			}
			catch( ... )
			{
				( new os::Alert( "Error: Failed to create file!", os::String ().Format( "Failed to create '%s'", cPath.c_str(  ) ), 0, "Sorry!", NULL ) )->Go( NULL );
			}

			psNode->m_pcRequester = NULL;

			if ( psNode->m_bDone == false )
			{
				Message cCancelMsg( ID_CANCEL_DOWNLOAD );

				cCancelMsg.AddPointer( "node", psNode );

				psNode->m_pcProgressDlg = new DownloadProgress( os::Rect( 100, 100, 449, 349 ), os::String ().Format( "Download: %s", cPath.c_str(  ) ), psNode->m_cURL, cPath, psNode->m_nContentSize, psNode->m_nStartTime, os::Messenger( this ), cCancelMsg );

				psNode->m_pcProgressDlg->Show();
			}
			else
			{
				delete psNode->m_pcTmpFile;
				delete psNode->m_pcFile;
				delete psNode;
			}
			break;
		}
	case ID_CANCEL_DOWNLOAD:
		{
			DownloadNode *psNode = NULL;

			pcMsg->FindPointer( "node", ( void ** )&psNode );

			if ( psNode == NULL )
			{
				break;	// Should never happen
			}
			if ( psNode->m_pcProgressDlg != NULL )
			{
				psNode->m_pcProgressDlg->Terminate();
				psNode->m_pcProgressDlg = NULL;
			}
			if ( psNode->m_cPath.empty() == false )
			{
				unlink( psNode->m_cPath.c_str() );
			}
			if ( psNode->m_bDone == false )
			{
				psNode->m_pcRequester = NULL;
				psNode->m_bCanceled = true;
				delete psNode->m_pcTmpFile;

				psNode->m_pcTmpFile = NULL;
				delete psNode->m_pcFile;

				psNode->m_pcFile = NULL;
			}
			else
			{
				delete psNode->m_pcTmpFile;
				delete psNode->m_pcFile;
				delete psNode;
			}
			break;
		}
	case ID_ABOUT:
		{
			os::Alert * pcAbout = new os::Alert( "About ABrowse", "ABrowse 0.4 Alpha3\nA KHTML based webbrowser\nWritten by Kurt Skauen, 2001\nWritten by Arno Klenke, 2004" "\n\nABrowse is released under the Gnu Public Licencse (GPL)\nPlease see the file COPYING, distributed with ABrowse, for\nmore information\n", os::Alert::ALERT_INFO, 0x00, "O.K", NULL );
			pcAbout->Go( new os::Invoker() );
			break;
		}
	case ID_ADD_BOOKMARK:
		{
			os::String zPath;
			bool bDummy;
			
			if( pcMsg->FindBool( "slave", &bDummy ) == 0 )
			{
				/* Update menu */
				m_pcBookmarkMenu->Reload();
				m_pcBookmarkMenu->SetTargetForItems( this );
				break;
			}
			
			if( pcMsg->FindString( "path", &zPath ) != 0 )
				break;
			
			
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( m_pcTabs->GetTabTitle( i ) == "Loading..." )
				break;
			os::String zFile = m_pcTabs->GetTabTitle( i );

			for ( i = 0; i < zFile.Length(); i++ )
				if ( zFile[i] == '/' )
					zFile[i] = '_';
			zFile = zPath + "/" + zFile;

			try
			{
				os::String zType = "text/x-bookmark";
				os::File cFile( zFile, O_RDWR | O_CREAT | O_TRUNC );
				cFile.Write( hview->GetCurrentURL().c_str(  ), hview->GetCurrentURL(  ).Length(  ) );
				cFile.WriteAttr( "os::MimeType", O_TRUNC, ATTR_TYPE_STRING, zType.c_str(), 0, zType.Length() );
				cFile.Flush();
			} catch( ... )
			{
			}
			/* Update menu */
			m_pcBookmarkMenu->Reload();
			m_pcBookmarkMenu->SetTargetForItems( this );
			
			/* Tell the other windows */
			
			for( uint i = 0; i < s_cWindows.size(); i++ )
			{
				if( s_cWindows[i] != this )
				{
					os::Message cMsg( *pcMsg );
					cMsg.AddBool( "slave", true );
					s_cWindows[i]->PostMessage( &cMsg, s_cWindows[i] );
				}
			}
			
			break;
		}
	case ID_MANAGE_BOOKMARKS:
		{
			os::String zPath;
			if( pcMsg->FindString( "path", &zPath ) != 0 )
				break;
			/* Open a filebrowser window */
			
			if ( fork() == 0 )
			{
				set_thread_priority( -1, 0 );
				execlp( "/system/bin/FileBrowser", "/system/bin/FileBrowser", zPath.c_str(), NULL );
			}
			break;
		}
	case ID_GO_BOOKMARK:
		{
			os::String zPath;
			if( pcMsg->FindString( "file/path", &zPath ) != 0 )
				break;
			char zBuffer[PATH_MAX];
			memset( zBuffer, 0, sizeof( zBuffer ) );
			try
			{
				os::File cFile( zPath );
				if( cFile.Read( zBuffer, PATH_MAX ) <= 0 )
					break;
				OpenURLRequest( zBuffer, KParts::URLArgs() );
			} catch( ... ) {
			}
			break;
		}
	case os::M_NODE_MONITOR:
		{
			
			break;
		}
	case ID_POPUP_CLOSED:
		{
			break;
		}
	default:
		Window::HandleMessage( pcMsg );
		break;
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool BrowserWindow::OkToQuit( void )
{
	if ( m_pcSearchDialog != NULL )
	{
		if ( m_pcSearchDialog->SafeLock() >= 0 )
		{
			m_pcSearchDialog->PostMessage( M_QUIT );
			m_pcSearchDialog->Unlock();
			m_pcSearchDialog = NULL;
		}
	}
	if ( s_cWindows.size() == 1 )
	{
		if ( m_bSaveSize )
		{
			char *pzHome = getenv( "HOME" );

			if ( NULL != pzHome )
			{
				os::Settings * pcSettings = new os::Settings();
				pcSettings->AddRect( "frame", GetFrame() );
				pcSettings->Save();
				delete( pcSettings );
			}
		}
		Application::GetInstance()->PostMessage( M_TERMINATE );
	}
	return ( true );
}




//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

BrowserApp::BrowserApp( const char *pzName, int argc, char *argv[] ):Application( pzName )
{
//    signal( SIGCHLD, SIG_IGN );

	
	/* Register types */
	m_pcManager = NULL;
	try
	{
		char zPath[PATH_MAX];
		memset( zPath, 0, sizeof( zPath ) );
		m_pcManager = os::RegistrarManager::Get();
		
		m_pcManager->RegisterType( "text/x-bookmark", "Bookmark" );
		m_pcManager->RegisterTypeIconFromRes( "text/x-bookmark", "text_bookmark.png" );
		m_pcManager->RegisterAsTypeHandler( "text/x-bookmark" );
		
		m_pcManager->RegisterType( "text/html", "HTML Page" );
		m_pcManager->RegisterTypeExtension( "text/html", "htm" );
		m_pcManager->RegisterTypeExtension( "text/html", "html" );
		m_pcManager->RegisterTypeIconFromRes( "text/html", "text_html.png" );
		m_pcManager->RegisterAsTypeHandler( "text/html" );
	} catch( ... ) {}

	GlobalMutex::Lock();
	SetMutex( GlobalMutex::GetMutex() );
	
	g_pcKApp = new KApplication( argc, argv, "abrowse" );

	/* Create bookmarks folder */
	char *pzHome = getenv( "HOME" );

	if ( NULL != pzHome )
	{
		try
		{
			os::Directory cDir( pzHome );
			os::Directory cBookmarkDir;
			cDir.CreateDirectory( "Bookmarks", &cBookmarkDir, cDir.GetMode() );
		}
		catch( ... )
		{
		}
	}

	// kjs_lib_count++; // ### hack, prevent khtmlpart from deleting the js klibrary object
	KLibLoader::self()->library( "kjs_html", KLibLoader::CreateStaticWrapper )->registerSymbol( "kjs_html_init", ( void * )&kjs_html_init );
	KLibLoader::self()->library( "libkhtml", KLibLoader::CreateStaticWrapper )->registerSymbol( "init_libkhtml", ( void * )&init_libkhtml );


	/* Load cookies */
	os::Path cCookieFile( pzHome );
	cCookieFile.Append( "Settings/Abrowse/cookies" );
	g_cCookieJar.loadCookies( cCookieFile.GetPath().c_str() );

	BrowserWindow *pcWindow = new BrowserWindow( Rect( 50, 20, 600, 480 ), true );

	KParts::URLArgs cArgs;

	/* Open the given or the default page */
	try
	{
		os::String cDefaultPath;
		bool bIsBookmark = false;
		if( argc > 1 )
		{
			/* Check if the parameter is a bookmark */
			os::String zPath( argv[1] );
			os::Path cPath( argv[1] );
			if( zPath.Length() > 5  && zPath.substr( 0, 4 ) == "http" )
				cDefaultPath = zPath;
			else
				cDefaultPath = cPath.GetPath();
			try 
			{
				char zAttrib[255];
				memset( zAttrib, 0, sizeof( zAttrib ) );
				os::File cFile( cDefaultPath );
				if( cFile.ReadAttr( "os::MimeType", ATTR_TYPE_STRING, zAttrib, 0, 255 ) > 0 )
					if( strcmp( zAttrib, "text/x-bookmark" ) == 0 )
						bIsBookmark = true;
			} catch( ... )
			{
			}
		} else {
		
			Directory cDir( open_image_file( IMGFILE_BIN_DIR ) );
			cDir.GetPath( &cDefaultPath );
			cDefaultPath = os::String ( "file:" ) + cDefaultPath + os::String ( "/docs/startup.html" );
		}
		if( bIsBookmark )
		{
			os::Message* pcMsg = new os::Message( ID_GO_BOOKMARK );
			pcMsg->AddString( "file/path", cDefaultPath );
			pcWindow->PostMessage( pcMsg, pcWindow );
		}
		else
			pcWindow->OpenURL( cDefaultPath.str(), cArgs );
	} catch( ... )
	{
	}

	pcWindow->Show();
	pcWindow->MakeFocus();
	AddTimer( this, 1, 50000, false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

BrowserApp::~BrowserApp()
{
	/* Wait until all windows have closed */
	Lock();
	while( BrowserWindow::s_cWindows.size() > 0 )
	{
		Unlock();
		snooze( 10000 );
		Lock();
	}

	Unlock();

	khtml::Cache::statistics();
	
	/* Save cookies */
	char *pzHome = getenv( "HOME" );
	os::Path cCookieFile( pzHome );
	cCookieFile.Append( "Settings/Abrowse/cookies" );
	g_cCookieJar.saveCookies( cCookieFile.GetPath().c_str() );
	
	/* Delete everything */
	khtml::Cache::clear();
	khtml::CSSStyleSelector::clear();
	khtml::RenderStyle::cleanup();
	
	delete( g_pcKApp );

	if( m_pcManager )
		m_pcManager->Put();
	
}

bool BrowserApp::OkToQuit()
{
	if( BrowserWindow::s_cWindows.size() == 0 )
		return( true );
	
	/* Delete window from list */
	Lock();

	for( uint i = 0; i < BrowserWindow::s_cWindows.size(); i++ ) {
		BrowserWindow::s_cWindows[i]->PostMessage( os::M_QUIT );
	}
	Unlock();
	return( false );
}

void BrowserApp::TimerTick( int /*nID */  )
{
	khtml::Loader::Run();
}

void print_msg( QtMsgType, const char *pzMsg )
{
	printf( "QWarning: %s\n", pzMsg );
}


int main( int argc, char *argv[] )
{
	BrowserApp *pcApp;
	
	qInstallMsgHandler( print_msg );

	pcApp = new BrowserApp( "application/x-vnd.Syllable-Abrowse", argc, argv );

	pcApp->Run();
	
	
	return ( 0 );
}































