
namespace os
{

	class BrowserWindow:public Window, public KParts::BrowserCallback
	{
	public:
		BrowserWindow( const Rect & cFrame, bool bLoadPos );
		 ~BrowserWindow();
		static os::Point GetNewWinPosition();
		void OpenURL( const std::string & cURL, const KParts::URLArgs & cArgs );
		void CreateNewTab();
		KParts::ReadOnlyPart * CreateNewTab( const std::string & cURL, const KParts::URLArgs & cArgs );
		void SwitchTab();
		void DeleteTab();
		virtual void HandleMessage( Message * pcMsg );

		// From Window:
		bool OkToQuit();

// From BrowserCallback:
		virtual void EnableAction( const char * /*name */ , bool /*enabled */  )
		{
		}
		virtual void OpenURLRequest( const std::string & url, const KParts::URLArgs & args );
		virtual void OpenURLNotify();
		virtual void Completed();
		virtual void SetLocationBarURL( const std::string & url );
		virtual void CreateNewWindow( const std::string & url, const KParts::URLArgs & args );
		virtual KParts::ReadOnlyPart * CreateNewWindow( const std::string & url, const KParts::URLArgs & args, const KParts::WindowArgs & windowArgs );
		virtual void PopupMenu( KParts::BrowserExtension * pcSource, const std::string & cURL );
		virtual void LoadingProgress( int percent );
		virtual void SpeedProgress( int bytesPerSecond );
		virtual void InfoMessage( const std::string & cString );
		virtual void SetWindowCaption( KParts::ReadOnlyPart * pcPart, const std::string & caption );
		virtual void SetStatusBarText( const std::string & text );
		virtual void SelectionInfo( const std::string & text );

		virtual bool BeginDownload( KParts::DownloadStream * psStream );
		virtual bool HandleDownloadData( KParts::DownloadStream * psStream, const void *pData, int nLen );
		virtual void DownloadFinished( KParts::DownloadStream * psStream, int nError );

		// New members:

		TextView *GetURLView();
		KHTMLPart *GetPart();

	private:
      	static std::vector<BrowserWindow*> s_cWindows;

		void UpdateHistory();
		void AddHistoryEntry();
		void GoHistory( int nStep, bool bButtonsOnly );

		std::string m_cSelectedText;
		std::string m_cLastSearchStr;
		bool m_bSearchCaseSensitive;
		LayoutView *m_pcFrame;
		ToolBar *m_pcToolBar;
		StatusBar *m_pcStatusBar;
		TextView *m_pcURLView;

		os::Menu* m_pcMenuBar;
		os::Menu * m_pcPopupMenu;
		BookmarksMenu * m_pcBookmarkMenu;
		os::TabView * m_pcTabs;
		FindDialog *m_pcSearchDialog;
		bool m_bSaveSize;
		friend class BrowserApp;
	};

	class BrowserApp:public Application
	{
	      public:
		BrowserApp( const char *pzName, int argc, char *argv[] );
		 ~BrowserApp();

		// From Handler:
		bool OkToQuit();
		virtual void TimerTick( int /*nID */  );
		private:
			
			os::RegistrarManager* m_pcManager;
	};

}

